
///////////////////////////////////////////////////////////////////////////////
//
//	areas.cpp
//

#include "areas.h"
#include "project.h"
#include "id.h"
#include "undo.h"

static const int area_type_count=7;
static const struct{
	BYTE code;
	char *name;
}area_type[area_type_count] = {
	{ 0x04, "Cranky's Cabin" },
	{ 0x06, "Funky's Flights" },
	{ 0x08, "Swanky's Quiz" },
	{ 0x0A, "Kong Kollege" },
	{ 0x0C, "Klubba" },
	{ 0x0E, "Normal Level" },
	{ 0x10, "Boss Stage" },
};


class AreaInfo_UndoMethod : public UndoMethod{
	ProjectManager *project;
	AreasManager *areas;
	AreaDataEx data_before;
	AreaDataEx data_after;
	BANK bank;
	SADRS adrs;
	int active_world_index;
	int area_index;
private:
	LPCSTR text_undo() const{ return "Area Setting"; }
	LPCSTR text_redo() const{ return "Area Setting"; }
	void undo(){
		common(data_before);
	}
	void redo(){
		common(data_after);
	}
	void common(const AreaDataEx &data){
		data.store(project, bank, adrs);
		areas->command(IDM_LEVEL_WLDCOMP, 0);		// GAݒ_CAOJ
		SendDlgItemMessage(areas->get_hWnd(), IDC_LIST0, LB_SETCURSEL, active_world_index, 0);
		areas->world_change(active_world_index);
		SendDlgItemMessage(areas->get_hWnd(), IDC_LIST1, LB_SETCURSEL, area_index, 0);
		areas->area_change(area_index);
	}
public:
	AreaInfo_UndoMethod(ProjectManager *_project, AreasManager *_areas, BANK _bank, SADRS _adrs,
		const AreaDataEx &_data_bef, const AreaDataEx &_data_aft, int _active_world_index, int _area_index)
		: project(_project), areas(_areas), bank(_bank), adrs(_adrs), data_before(_data_bef), data_after(_data_aft),
		active_world_index(_active_world_index), area_index(_area_index)
	{}
	~AreaInfo_UndoMethod(){}
};




AreasManager::AreasManager(ProjectManager *_project) : project(_project){
	area_count = GetPrivateProfileInt("Areas", "Count", 120, RomInfoFileName);
	if (area_count<=0 || area_count>=256) area_count = 120;
	area_name = new char[area_count][256];
	for (int i=0; i<area_count; i++){
		char s[16];
		sprintf_s(s, sizeof s, "%02X", i);
		GetPrivateProfileString("AreaName", s, "?", area_name[i], 256, RomInfoFileName);
	}
	pAreaInfoAdrs = (DWORD)GetPrivateProfileInt("Areas", "AreaInfoAddress", 0x34CBDB, RomInfoFileName);
	pAreaAvailableBegin	= (WORD)GetPrivateProfileInt("Areas", "AvailableBegin",	0xCD43, RomInfoFileName);
	pAreaAvailableEnd	= (WORD)GetPrivateProfileInt("Areas", "AvailableEnd",	0xDA77, RomInfoFileName);
}


AreasManager::~AreasManager(){
	delete[] area_name;
}


BOOL AreasManager::create(){
	return TRUE;
}

BOOL AreasManager::command(WPARAM wParam, LPARAM lParam){
	if (LOWORD(wParam) == IDM_LEVEL_WLDCOMP){
		if (!bValid){
			create_dialog(project->get_hWnd());
		}
		area_change(SendDlgItemMessage(hWnd, IDC_LIST1, LB_GETCURSEL, 0, 0));
		ShowWindow(hWnd, SW_SHOW);
		SetFocus(GetDlgItem(hWnd, IDC_LIST0));
		return TRUE;
	}
	return FALSE;
}

BOOL AreasManager::translate_dialog_message(MSG *msg){
	if (bValid){
		return IsDialogMessage(hWnd, msg);
	}
	return FALSE;
}

void AreasManager::set_state(BOOL bEnabled){
	HMENU hmMain = GetMenu(project->get_hWnd());
	UINT uState = (bEnabled ? MF_ENABLED : MF_GRAYED) | MF_BYCOMMAND;
	EnableMenuItem(hmMain, IDM_LEVEL_WLDCOMP, uState);
	return;
}


BOOL AreasManager::Proc(HWND hDlg, UINT msg, WPARAM wParam, LPARAM lParam){
	switch (msg){
		case WM_INITDIALOG:
			init();
			return TRUE;
		case WM_COMMAND:
			switch (LOWORD(wParam)){
				case IDC_LIST0:
					switch (HIWORD(wParam)){
						case LBN_SELCHANGE:
							world_change(SendDlgItemMessage(hWnd, IDC_LIST0, LB_GETCURSEL, 0, 0));
							return TRUE;
					}
					break;
				case IDC_LIST1:
					switch (HIWORD(wParam)){
						case LBN_SELCHANGE:
							area_change(SendDlgItemMessage(hWnd, IDC_LIST1, LB_GETCURSEL, 0, 0));
							return TRUE;
					}
					break;
				case IDC_COMBO2:
					if (HIWORD(wParam) == CBN_SELCHANGE){
						SetDlgItemText(hWnd, IDC_STATIC2, "Type:");
						return TRUE;
					}
					break;
				case IDC_EDIT3:
					if (HIWORD(wParam) == EN_CHANGE && !unable_edit_change){
						SetDlgItemText(hWnd, IDC_STATIC3, "X:");
						return TRUE;
					}
					break;
				case IDC_EDIT4:
					if (HIWORD(wParam) == EN_CHANGE && !unable_edit_change){
						SetDlgItemText(hWnd, IDC_STATIC4, "Y:");
						return TRUE;
					}
					break;
				case IDC_EDIT5:
					if (HIWORD(wParam) == EN_CHANGE && !unable_edit_change){
						SetDlgItemText(hWnd, IDC_STATIC5, "Icon:");
						return TRUE;
					}
					break;
				case IDC_EDIT7:
					if (HIWORD(wParam) == EN_CHANGE && !unable_edit_change){
						SetDlgItemText(hWnd, IDC_STATIC7, "Level:");
						return TRUE;
					}
					break;
				case IDC_EDIT6:
					if (HIWORD(wParam) == EN_CHANGE && !unable_edit_change){
						SetDlgItemText(hWnd, IDC_STATIC6, "Name:");
						return TRUE;
					}
					break;
				case IDC_EDIT8:
					if (HIWORD(wParam) == EN_CHANGE && !unable_edit_change){
						SetDlgItemText(hWnd, IDC_STATIC8, "Parameters:");
						return TRUE;
					}
					break;
				case IDC_CHECK0:
					EnableWindow(GetDlgItem(hWnd, IDC_STATIC11), IsDlgButtonChecked(hWnd, LOWORD(wParam)));
					EnableWindow(GetDlgItem(hWnd, IDC_STATIC12), IsDlgButtonChecked(hWnd, LOWORD(wParam)));
					EnableWindow(GetDlgItem(hWnd, IDC_EDIT11), IsDlgButtonChecked(hWnd, LOWORD(wParam)));
					EnableWindow(GetDlgItem(hWnd, IDC_EDIT12), IsDlgButtonChecked(hWnd, LOWORD(wParam)));
				case IDC_CHECK1:
				case IDC_EDIT11:
				case IDC_EDIT12:
					if ((LOWORD(wParam)>=IDC_CHECK0&&LOWORD(wParam)<=IDC_CHECK7) || (HIWORD(wParam)==EN_CHANGE&&!unable_edit_change)){
						SetDlgItemText(hWnd, IDC_STATIC10, "Left");
					}
					return TRUE;
				case IDC_CHECK2:
					EnableWindow(GetDlgItem(hWnd, IDC_STATIC15), IsDlgButtonChecked(hWnd, LOWORD(wParam)));
					EnableWindow(GetDlgItem(hWnd, IDC_STATIC16), IsDlgButtonChecked(hWnd, LOWORD(wParam)));
					EnableWindow(GetDlgItem(hWnd, IDC_EDIT15), IsDlgButtonChecked(hWnd, LOWORD(wParam)));
					EnableWindow(GetDlgItem(hWnd, IDC_EDIT16), IsDlgButtonChecked(hWnd, LOWORD(wParam)));
				case IDC_CHECK3:
				case IDC_EDIT15:
				case IDC_EDIT16:
					if ((LOWORD(wParam)>=IDC_CHECK0&&LOWORD(wParam)<=IDC_CHECK7) || (HIWORD(wParam)==EN_CHANGE&&!unable_edit_change)){
						SetDlgItemText(hWnd, IDC_STATIC14, "Right");
					}
					return TRUE;
				case IDC_CHECK4:
					EnableWindow(GetDlgItem(hWnd, IDC_STATIC19), IsDlgButtonChecked(hWnd, LOWORD(wParam)));
					EnableWindow(GetDlgItem(hWnd, IDC_STATIC1A), IsDlgButtonChecked(hWnd, LOWORD(wParam)));
					EnableWindow(GetDlgItem(hWnd, IDC_EDIT19), IsDlgButtonChecked(hWnd, LOWORD(wParam)));
					EnableWindow(GetDlgItem(hWnd, IDC_EDIT1A), IsDlgButtonChecked(hWnd, LOWORD(wParam)));
				case IDC_CHECK5:
				case IDC_EDIT19:
				case IDC_EDIT1A:
					if ((LOWORD(wParam)>=IDC_CHECK0&&LOWORD(wParam)<=IDC_CHECK7) || (HIWORD(wParam)==EN_CHANGE&&!unable_edit_change)){
						SetDlgItemText(hWnd, IDC_STATIC18, "Up");
					}
					return TRUE;
				case IDC_CHECK6:
					EnableWindow(GetDlgItem(hWnd, IDC_STATIC1D), IsDlgButtonChecked(hWnd, LOWORD(wParam)));
					EnableWindow(GetDlgItem(hWnd, IDC_STATIC1E), IsDlgButtonChecked(hWnd, LOWORD(wParam)));
					EnableWindow(GetDlgItem(hWnd, IDC_EDIT1D), IsDlgButtonChecked(hWnd, LOWORD(wParam)));
					EnableWindow(GetDlgItem(hWnd, IDC_EDIT1E), IsDlgButtonChecked(hWnd, LOWORD(wParam)));
				case IDC_CHECK7:
				case IDC_EDIT1D:
				case IDC_EDIT1E:
					if ((LOWORD(wParam)>=IDC_CHECK0&&LOWORD(wParam)<=IDC_CHECK7) || (HIWORD(wParam)==EN_CHANGE&&!unable_edit_change)){
						SetDlgItemText(hWnd, IDC_STATIC1C, "Down");
					}
					return TRUE;
				case IDOK:
					apply();
					return TRUE;
				case IDCANCEL:
					ShowWindow(hWnd, SW_HIDE);
					return TRUE;
			}
			break;
	}
	return FALSE;
}


void AreasManager::init(){
	char s[256];

	unable_edit_change = true;

	// GfBbgRg[
	SendDlgItemMessage(hWnd, IDC_EDIT5, EM_SETLIMITTEXT, 2, 0);
	SendDlgItemMessage(hWnd, IDC_EDIT7, EM_SETLIMITTEXT, 4, 0);
	SendDlgItemMessage(hWnd, IDC_EDIT8, EM_SETLIMITTEXT, 2, 0);
	SendDlgItemMessage(hWnd, IDC_EDIT11, EM_SETLIMITTEXT, 4, 0);
	SendDlgItemMessage(hWnd, IDC_EDIT12, EM_SETLIMITTEXT, 4, 0);
	SendDlgItemMessage(hWnd, IDC_EDIT15, EM_SETLIMITTEXT, 4, 0);
	SendDlgItemMessage(hWnd, IDC_EDIT16, EM_SETLIMITTEXT, 4, 0);
	SendDlgItemMessage(hWnd, IDC_EDIT19, EM_SETLIMITTEXT, 4, 0);
	SendDlgItemMessage(hWnd, IDC_EDIT1A, EM_SETLIMITTEXT, 4, 0);
	SendDlgItemMessage(hWnd, IDC_EDIT1D, EM_SETLIMITTEXT, 4, 0);
	SendDlgItemMessage(hWnd, IDC_EDIT1E, EM_SETLIMITTEXT, 4, 0);

	// R{{bNX
	HWND hwCombo = GetDlgItem(hWnd, IDC_COMBO2);
	for (int i=0; i<area_type_count; i++){
		sprintf_s(s, sizeof s, "%02X: %s", area_type[i].code, area_type[i].name);
		SendMessage(hwCombo, CB_INSERTSTRING, (WPARAM)i, (LPARAM)s);
	}

	// [hXg
	HWND hwWorldList = GetDlgItem(hWnd, IDC_LIST0);
	SendMessage(hwWorldList, LB_INSERTSTRING, (WPARAM)0, (LPARAM)"Worlds");
	int world_count = GetPrivateProfileInt("Worlds", "Count", 15, RomInfoFileName);
	if (world_count<0 || world_count>=256) world_count=15;
	for (int i=0; i<world_count; i++){
		sprintf_s(s, sizeof s, "%02X", i);
		GetPrivateProfileString("WorldName", s, "?", s+4, (sizeof s)-4, RomInfoFileName);
		s[2] = ':', s[3] = ' ';
		SendMessage(hwWorldList, LB_INSERTSTRING, (WPARAM)(i+1), (LPARAM)s);
	}

	unable_edit_change = false;
	return;
}

void AreasManager::world_change(int index){
	char s[256];
	bool *areas = new bool[area_count];
	if (index == 0){
		// SẴGAΏۂƂ
		for (int i=0; i<area_count; i++){
			areas[i] = true;
		}
	}else{
		// [h̎QƂGÂݑΏۂƂ
		for (int i=0; i<area_count; i++){
			areas[i] = false;
		}
		int world_num = index - 1;
		LADRS pWorldInfoAdrs = (DWORD)GetPrivateProfileInt("Worlds", "WorldInfoAddress", 0x34C204, RomInfoFileName);
		SADRS world_adrs = project->read_sadrs(pWorldInfoAdrs.bank, pWorldInfoAdrs.adrs + world_num*2);
		WORD world_area_count = project->read_word(pWorldInfoAdrs.bank, world_adrs + 0x0A);
		for (int i=0; i<world_area_count; i++){
			WORD world_area_num = project->read_word(pWorldInfoAdrs.bank, world_adrs + 0x0E + i*2);
			if (world_area_num < area_count){
				areas[world_area_num] = true;
			}
		}
	}

	HWND hwAreaList = GetDlgItem(hWnd, IDC_LIST1);
	SendMessage(hwAreaList, WM_SETREDRAW, FALSE, 0);
	SendMessage(hwAreaList, LB_RESETCONTENT, 0, 0);
	int n=0;
	for (int i=0; i<area_count; i++){
		if (areas[i]){
			sprintf_s(s, sizeof s, "%02X: %s", i, area_name[i]);
			SendMessage(hwAreaList, LB_INSERTSTRING, (WPARAM)n, (LPARAM)s);
			n++;
		}
	}
	SendMessage(hwAreaList, WM_SETREDRAW, TRUE, 0);
	area_change(-1);
	delete[] areas;
}

void AreasManager::area_change(int index){
	char s[1024];
	int area_num;
	if (index >= 0){
		SendDlgItemMessage(hWnd, IDC_LIST1, LB_GETTEXT, (WPARAM)index, (LPARAM)s);
		s[2] = '\0';
		if (sscanf_s(s, "%x", &area_num) < 0) return;
		if (area_num < 0 || area_num >= area_count) return;
	}
	unable_edit_change = true;
	AreaDataEx data;
	if (index >= 0){
		data.load(project, pAreaInfoAdrs.bank, pAreaInfoAdrs.adrs + area_num*3);
	}else{
		data.bValid = false;
	}
	SetDlgItemText(hWnd, IDC_STATIC2, "Type:");
	SetDlgItemText(hWnd, IDC_STATIC3, "X:");
	SetDlgItemText(hWnd, IDC_STATIC4, "Y:");
	SetDlgItemText(hWnd, IDC_STATIC5, "Icon:");
	SetDlgItemText(hWnd, IDC_STATIC7, "Level:");
	SetDlgItemText(hWnd, IDC_STATIC6, "Name:");
	SetDlgItemText(hWnd, IDC_STATIC8, "Parameters:");
	SetDlgItemText(hWnd, IDC_STATIC10, "Left");
	SetDlgItemText(hWnd, IDC_STATIC14, "Right");
	SetDlgItemText(hWnd, IDC_STATIC18, "Up");
	SetDlgItemText(hWnd, IDC_STATIC1C, "Down");
	EnableWindow(GetDlgItem(hWnd, IDC_STATIC2), data.bValid? TRUE : FALSE);
	EnableWindow(GetDlgItem(hWnd, IDC_COMBO2), data.bValid? TRUE : FALSE);
	EnableWindow(GetDlgItem(hWnd, IDC_STATIC3), data.bValid? TRUE : FALSE);
	EnableWindow(GetDlgItem(hWnd, IDC_EDIT3), data.bValid? TRUE : FALSE);
	EnableWindow(GetDlgItem(hWnd, IDC_STATIC4), data.bValid? TRUE : FALSE);
	EnableWindow(GetDlgItem(hWnd, IDC_EDIT4), data.bValid? TRUE : FALSE);
	EnableWindow(GetDlgItem(hWnd, IDC_STATIC5), data.bValid? TRUE : FALSE);
	EnableWindow(GetDlgItem(hWnd, IDC_EDIT5), data.bValid? TRUE : FALSE);
	EnableWindow(GetDlgItem(hWnd, IDC_STATIC6), data.bValid? TRUE : FALSE);
	EnableWindow(GetDlgItem(hWnd, IDC_EDIT6), data.bValid? TRUE : FALSE);
	EnableWindow(GetDlgItem(hWnd, IDC_STATIC7), data.bValid? TRUE : FALSE);
	EnableWindow(GetDlgItem(hWnd, IDC_EDIT7), data.bValid? TRUE : FALSE);
	EnableWindow(GetDlgItem(hWnd, IDC_STATIC8), data.bValid? TRUE : FALSE);
	EnableWindow(GetDlgItem(hWnd, IDC_EDIT8), data.bValid? TRUE : FALSE);
	EnableWindow(GetDlgItem(hWnd, IDC_STATIC10), data.bValid? TRUE : FALSE);
	EnableWindow(GetDlgItem(hWnd, IDC_STATIC14), data.bValid? TRUE : FALSE);
	EnableWindow(GetDlgItem(hWnd, IDC_STATIC18), data.bValid? TRUE : FALSE);
	EnableWindow(GetDlgItem(hWnd, IDC_STATIC1C), data.bValid? TRUE : FALSE);
	EnableWindow(GetDlgItem(hWnd, IDC_CHECK0), data.bValid? TRUE : FALSE);
	EnableWindow(GetDlgItem(hWnd, IDC_CHECK1), data.bValid? TRUE : FALSE);
	EnableWindow(GetDlgItem(hWnd, IDC_CHECK2), data.bValid? TRUE : FALSE);
	EnableWindow(GetDlgItem(hWnd, IDC_CHECK3), data.bValid? TRUE : FALSE);
	EnableWindow(GetDlgItem(hWnd, IDC_CHECK4), data.bValid? TRUE : FALSE);
	EnableWindow(GetDlgItem(hWnd, IDC_CHECK5), data.bValid? TRUE : FALSE);
	EnableWindow(GetDlgItem(hWnd, IDC_CHECK6), data.bValid? TRUE : FALSE);
	EnableWindow(GetDlgItem(hWnd, IDC_CHECK7), data.bValid? TRUE : FALSE);
	EnableWindow(GetDlgItem(hWnd, IDOK), data.bValid? TRUE : FALSE);
	if (data.bValid){
		int area_type_index = -1;
		for (int i=0; i<area_type_count; i++){
			if (data.type == area_type[i].code){
				area_type_index = i;
				break;
			}
		}
		if (area_type_index == -1){
			sprintf_s(s, sizeof s, "Unexpected type of area: %02X", data.type);
			MessageBox(hWnd, s, "Error", MB_OK | MB_ICONEXCLAMATION);
		}
		SendDlgItemMessage(hWnd, IDC_COMBO2, CB_SETCURSEL, (WPARAM)area_type_index, 0);
		sprintf_s(s, sizeof s, "%d", data.pos_x);
		SetDlgItemText(hWnd, IDC_EDIT3, s);
		sprintf_s(s, sizeof s, "%d", data.pos_y);
		SetDlgItemText(hWnd, IDC_EDIT4, s);
		sprintf_s(s, sizeof s, "%02X", data.icon);
		SetDlgItemText(hWnd, IDC_EDIT5, s);
		project->characters.to_shift_jis(s, data.text.string, data.text.length);
		SetDlgItemText(hWnd, IDC_EDIT6, s);
		sprintf_s(s, sizeof s, "%04X", data.level_num);
		SetDlgItemText(hWnd, IDC_EDIT7, s);
		sprintf_s(s, sizeof s, "%02X", data.text.param);
		SetDlgItemText(hWnd, IDC_EDIT8, s);
		CheckDlgButton(hWnd, IDC_CHECK0, data.road_left.bValid? TRUE : FALSE);
		CheckDlgButton(hWnd, IDC_CHECK1, data.road_left.bParam? TRUE : FALSE);
		CheckDlgButton(hWnd, IDC_CHECK2, data.road_right.bValid? TRUE : FALSE);
		CheckDlgButton(hWnd, IDC_CHECK3, data.road_right.bParam? TRUE : FALSE);
		CheckDlgButton(hWnd, IDC_CHECK4, data.road_up.bValid? TRUE : FALSE);
		CheckDlgButton(hWnd, IDC_CHECK5, data.road_up.bParam? TRUE : FALSE);
		CheckDlgButton(hWnd, IDC_CHECK6, data.road_down.bValid? TRUE : FALSE);
		CheckDlgButton(hWnd, IDC_CHECK7, data.road_down.bParam? TRUE : FALSE);
		EnableWindow(GetDlgItem(hWnd, IDC_STATIC11), data.road_left.bValid? TRUE : FALSE);
		EnableWindow(GetDlgItem(hWnd, IDC_STATIC12), data.road_left.bValid? TRUE : FALSE);
		EnableWindow(GetDlgItem(hWnd, IDC_EDIT11), data.road_left.bValid? TRUE : FALSE);
		EnableWindow(GetDlgItem(hWnd, IDC_EDIT12), data.road_left.bValid? TRUE : FALSE);
		EnableWindow(GetDlgItem(hWnd, IDC_STATIC15), data.road_right.bValid? TRUE : FALSE);
		EnableWindow(GetDlgItem(hWnd, IDC_STATIC16), data.road_right.bValid? TRUE : FALSE);
		EnableWindow(GetDlgItem(hWnd, IDC_EDIT15), data.road_right.bValid? TRUE : FALSE);
		EnableWindow(GetDlgItem(hWnd, IDC_EDIT16), data.road_right.bValid? TRUE : FALSE);
		EnableWindow(GetDlgItem(hWnd, IDC_STATIC19), data.road_up.bValid? TRUE : FALSE);
		EnableWindow(GetDlgItem(hWnd, IDC_STATIC1A), data.road_up.bValid? TRUE : FALSE);
		EnableWindow(GetDlgItem(hWnd, IDC_EDIT19), data.road_up.bValid? TRUE : FALSE);
		EnableWindow(GetDlgItem(hWnd, IDC_EDIT1A), data.road_up.bValid? TRUE : FALSE);
		EnableWindow(GetDlgItem(hWnd, IDC_STATIC1D), data.road_down.bValid? TRUE : FALSE);
		EnableWindow(GetDlgItem(hWnd, IDC_STATIC1E), data.road_down.bValid? TRUE : FALSE);
		EnableWindow(GetDlgItem(hWnd, IDC_EDIT1D), data.road_down.bValid? TRUE : FALSE);
		EnableWindow(GetDlgItem(hWnd, IDC_EDIT1E), data.road_down.bValid? TRUE : FALSE);
		SetDlgItemText(hWnd, IDC_EDIT11, data.road_left.bValid? (sprintf_s(s, sizeof s, "%04X", data.road_left.area_num), s) : "");
		SetDlgItemText(hWnd, IDC_EDIT12, data.road_left.bValid? (sprintf_s(s, sizeof s, "%04X", data.road_left.road_num), s) : "");
		SetDlgItemText(hWnd, IDC_EDIT15, data.road_right.bValid? (sprintf_s(s, sizeof s, "%04X", data.road_right.area_num), s) : "");
		SetDlgItemText(hWnd, IDC_EDIT16, data.road_right.bValid? (sprintf_s(s, sizeof s, "%04X", data.road_right.road_num), s) : "");
		SetDlgItemText(hWnd, IDC_EDIT19, data.road_up.bValid? (sprintf_s(s, sizeof s, "%04X", data.road_up.area_num), s) : "");
		SetDlgItemText(hWnd, IDC_EDIT1A, data.road_up.bValid? (sprintf_s(s, sizeof s, "%04X", data.road_up.road_num), s) : "");
		SetDlgItemText(hWnd, IDC_EDIT1D, data.road_down.bValid? (sprintf_s(s, sizeof s, "%04X", data.road_down.area_num), s) : "");
		SetDlgItemText(hWnd, IDC_EDIT1E, data.road_down.bValid? (sprintf_s(s, sizeof s, "%04X", data.road_down.road_num), s) : "");
	}else{
		EnableWindow(GetDlgItem(hWnd, IDC_STATIC11), FALSE);
		EnableWindow(GetDlgItem(hWnd, IDC_STATIC12), FALSE);
		EnableWindow(GetDlgItem(hWnd, IDC_EDIT11), FALSE);
		EnableWindow(GetDlgItem(hWnd, IDC_EDIT12), FALSE);
		EnableWindow(GetDlgItem(hWnd, IDC_STATIC15), FALSE);
		EnableWindow(GetDlgItem(hWnd, IDC_STATIC16), FALSE);
		EnableWindow(GetDlgItem(hWnd, IDC_EDIT15), FALSE);
		EnableWindow(GetDlgItem(hWnd, IDC_EDIT16), FALSE);
		EnableWindow(GetDlgItem(hWnd, IDC_STATIC19), FALSE);
		EnableWindow(GetDlgItem(hWnd, IDC_STATIC1A), FALSE);
		EnableWindow(GetDlgItem(hWnd, IDC_EDIT19), FALSE);
		EnableWindow(GetDlgItem(hWnd, IDC_EDIT1A), FALSE);
		EnableWindow(GetDlgItem(hWnd, IDC_STATIC1D), FALSE);
		EnableWindow(GetDlgItem(hWnd, IDC_STATIC1E), FALSE);
		EnableWindow(GetDlgItem(hWnd, IDC_EDIT1D), FALSE);
		EnableWindow(GetDlgItem(hWnd, IDC_EDIT1E), FALSE);
	}
	unable_edit_change = false;
	return;
}

void AreasManager::apply(){
	char s[1024];
	int area_num;
	int area_index = SendDlgItemMessage(hWnd, IDC_LIST1, LB_GETCURSEL, 0, 0);
	if (area_index < 0){
		MessageBox(hWnd, "No Area is selected", "Error", MB_OK | MB_ICONEXCLAMATION);
		return;
	}
	SendDlgItemMessage(hWnd, IDC_LIST1, LB_GETTEXT, (WPARAM)area_index, (LPARAM)s);
	s[2] = '\0';
	if (sscanf_s(s, "%x", &area_num) < 0) return;
	if (area_num < 0 || area_num >= area_count) return;

	AreaDataEx data, data_org;
	data.load(project, pAreaInfoAdrs.bank, pAreaInfoAdrs.adrs + area_num*3);
	data_org = data;
	if (!data.bValid){
		MessageBox(hWnd, "Rewriting the data in this area is not allowed.", "Error", MB_OK | MB_ICONEXCLAMATION);
		return;
	}

	int val;
	val = SendDlgItemMessage(hWnd, IDC_COMBO2, CB_GETCURSEL, 0, 0);
	if (val == -1){
		MessageBox(hWnd, "No areas selected type.", "Error", MB_OK | MB_ICONEXCLAMATION);
		return;
	}
	data.type = area_type[val].code;
	GetDlgItemText(hWnd, IDC_EDIT3, s, sizeof s);
	if (sscanf_s(s, "%d", &val) < 0 || val < 0 || val >= 0x10000){
		MessageBox(hWnd, "X coordinate value is invalid.", "Error", MB_OK | MB_ICONEXCLAMATION);
		return;
	}
	data.pos_x = (WORD)val;
	GetDlgItemText(hWnd, IDC_EDIT4, s, sizeof s);
	if (sscanf_s(s, "%d", &val) < 0 || val < 0 || val >= 0x10000){
		MessageBox(hWnd, "Y coordinate value is invalid.", "Error", MB_OK | MB_ICONEXCLAMATION);
		return;
	}
	data.pos_y = (WORD)val;
	GetDlgItemText(hWnd, IDC_EDIT5, s, sizeof s);
	if (sscanf_s(s, "%x", &val) < 0 || val < 0 || val >= 0x100){
		MessageBox(hWnd, "Invalid icon value", "Error", MB_OK | MB_ICONEXCLAMATION);
		return;
	}
	data.icon = (BYTE)val;
	GetDlgItemText(hWnd, IDC_EDIT7, s, sizeof s);
	if (sscanf_s(s, "%x", &val) < 0 || val < 0 || val >= 0x10000){
		MessageBox(hWnd, "Invalid Stage Value", "Error", MB_OK | MB_ICONEXCLAMATION);
		return;
	}
	data.level_num = (BYTE)val;
	GetDlgItemText(hWnd, IDC_EDIT6, s, sizeof s);
	val = project->characters.from_shift_jis(s, data.text.string, 256);
	if (val < 0 || val > 256){
		MessageBox(hWnd, "Unable to change the level name", "Error", MB_OK | MB_ICONEXCLAMATION);
		return;
	}
	data.text.length = (BYTE)val;
	GetDlgItemText(hWnd, IDC_EDIT8, s, sizeof s);
	if (sscanf_s(s, "%x", &val) < 0 || val < 0 || val >= 0x100){
		MessageBox(hWnd, "Invalid parameter", "Error", MB_OK | MB_ICONEXCLAMATION);
		return;
	}
	data.text.param = (BYTE)val;

	data.road_left.bValid	= IsDlgButtonChecked(hWnd, IDC_CHECK0) != FALSE;
	data.road_left.bParam	= IsDlgButtonChecked(hWnd, IDC_CHECK1) != FALSE;
	data.road_right.bValid	= IsDlgButtonChecked(hWnd, IDC_CHECK2) != FALSE;
	data.road_right.bParam	= IsDlgButtonChecked(hWnd, IDC_CHECK3) != FALSE;
	data.road_up.bValid		= IsDlgButtonChecked(hWnd, IDC_CHECK4) != FALSE;
	data.road_up.bParam		= IsDlgButtonChecked(hWnd, IDC_CHECK5) != FALSE;
	data.road_down.bValid	= IsDlgButtonChecked(hWnd, IDC_CHECK6) != FALSE;
	data.road_down.bParam	= IsDlgButtonChecked(hWnd, IDC_CHECK7) != FALSE;
	if (data.road_left.bValid){
		GetDlgItemText(hWnd, IDC_EDIT11, s, sizeof s);
		if (sscanf_s(s, "%x", &val) < 0 || val < 0 || val >= 0x10000){
			MessageBox(hWnd, "Invalid left destination value.", "Error", MB_OK | MB_ICONEXCLAMATION);
			return;
		}
		data.road_left.area_num = (WORD)val;
		GetDlgItemText(hWnd, IDC_EDIT12, s, sizeof s);
		if (sscanf_s(s, "%x", &val) < 0 || val < 0 || val >= 0x10000){
			MessageBox(hWnd, "Invalid left path value.", "Error", MB_OK | MB_ICONEXCLAMATION);
			return;
		}
		data.road_left.road_num = (WORD)val;
	}
	if (data.road_right.bValid){
		GetDlgItemText(hWnd, IDC_EDIT15, s, sizeof s);
		if (sscanf_s(s, "%x", &val) < 0 || val < 0 || val >= 0x10000){
			MessageBox(hWnd, "Invalid right destination value.", "Error", MB_OK | MB_ICONEXCLAMATION);
			return;
		}
		data.road_right.area_num = (WORD)val;
		GetDlgItemText(hWnd, IDC_EDIT16, s, sizeof s);
		if (sscanf_s(s, "%x", &val) < 0 || val < 0 || val >= 0x10000){
			MessageBox(hWnd, "Invalid right path value.", "Error", MB_OK | MB_ICONEXCLAMATION);
			return;
		}
		data.road_right.road_num = (WORD)val;
	}
	if (data.road_up.bValid){
		GetDlgItemText(hWnd, IDC_EDIT19, s, sizeof s);
		if (sscanf_s(s, "%x", &val) < 0 || val < 0 || val >= 0x10000){
			MessageBox(hWnd, "Invalid up destination value.", "Error", MB_OK | MB_ICONEXCLAMATION);
			return;
		}
		data.road_up.area_num = (WORD)val;
		GetDlgItemText(hWnd, IDC_EDIT1A, s, sizeof s);
		if (sscanf_s(s, "%x", &val) < 0 || val < 0 || val >= 0x10000){
			MessageBox(hWnd, "Invalid up path value.", "Error", MB_OK | MB_ICONEXCLAMATION);
			return;
		}
		data.road_up.road_num = (WORD)val;
	}
	if (data.road_down.bValid){
		GetDlgItemText(hWnd, IDC_EDIT1D, s, sizeof s);
		if (sscanf_s(s, "%x", &val) < 0 || val < 0 || val >= 0x10000){
			MessageBox(hWnd, "Invalid down destination value.", "Error", MB_OK | MB_ICONEXCLAMATION);
			return;
		}
		data.road_down.area_num = (WORD)val;
		GetDlgItemText(hWnd, IDC_EDIT1E, s, sizeof s);
		if (sscanf_s(s, "%x", &val) < 0 || val < 0 || val >= 0x10000){
			MessageBox(hWnd, "Invalid down path value.", "Error", MB_OK | MB_ICONEXCLAMATION);
			return;
		}
		data.road_down.road_num = (WORD)val;
	}

	// p\ȗ̈𒲂ׂ
	SADRS available_end = pAreaAvailableEnd;
	SADRS adrs_this = project->read_sadrs(pAreaInfoAdrs.bank, pAreaInfoAdrs.adrs + area_num*3 + 1);
	for (int i=0; i<area_count; i++){
		SADRS adrs = project->read_sadrs(pAreaInfoAdrs.bank, pAreaInfoAdrs.adrs + i*3 + 1);
		if (adrs > adrs_this && adrs < available_end){
			available_end = adrs;
		}
	}
	SADRS data_end_org = data_org.get_end(project, pAreaInfoAdrs.bank, pAreaInfoAdrs.adrs + area_num*3);
	SADRS data_end_new = data.get_end(project, pAreaInfoAdrs.bank, pAreaInfoAdrs.adrs + area_num*3);
	//sprintf_s(s, sizeof s, "end_org: %04X\r\nend_new: %04X\r\navailable_end: %04X", data_end_org, data_end_new, available_end);
	//MessageBox(hWnd, s, "", 0);
	if (data_end_new > available_end){
		sprintf_s(s, sizeof s, "Data size and fit into the available space.\r\n"
			"End of space available: %04X\r\nTermination of the original data: %04X\r\nTo end of data to write: %04X",
			available_end, data_end_org, data_end_new);
		MessageBox(hWnd, s, "G[", MB_OK | MB_ICONEXCLAMATION);
		return;
	}

	// f[^
	data.store(project, pAreaInfoAdrs.bank, pAreaInfoAdrs.adrs + area_num*3);

	// AhDobt@ɒǉ
	AreaInfo_UndoMethod *undo = new AreaInfo_UndoMethod(
		project, this, pAreaInfoAdrs.bank, pAreaInfoAdrs.adrs + area_num*3, data_org, data,
		(int)SendDlgItemMessage(hWnd, IDC_LIST0, LB_GETCURSEL, 0, 0), area_index);
	project->undo_buffer.add_method(undo);

	SetDlgItemText(hWnd, IDC_STATIC2, "Type:");
	SetDlgItemText(hWnd, IDC_STATIC3, "X:");
	SetDlgItemText(hWnd, IDC_STATIC4, "Y:");
	SetDlgItemText(hWnd, IDC_STATIC5, "Icon:");
	SetDlgItemText(hWnd, IDC_STATIC7, "Level:");
	SetDlgItemText(hWnd, IDC_STATIC6, "Name:");
	SetDlgItemText(hWnd, IDC_STATIC8, "Parameter:");
	SetDlgItemText(hWnd, IDC_STATIC10, "Left");
	SetDlgItemText(hWnd, IDC_STATIC14, "Right");
	SetDlgItemText(hWnd, IDC_STATIC18, "Up");
	SetDlgItemText(hWnd, IDC_STATIC1C, "Down");
	return;
}




//=================================================================================================

SADRS AreaDataEx::load(const ProjectManager *project, BANK bank, SADRS adrs){
	type			= project->read_byte(bank, adrs);
	SADRS pData		= project->read_sadrs(bank, adrs+1);
	if (pData == 0x0000){
		bValid = false;
		return 0x0000;
	}
	pos_x			= project->read_word(bank, pData);		pData += 2;
	pos_y			= project->read_word(bank, pData);		pData += 2;
	icon			= project->read_byte(bank, pData);		pData++;
	BYTE arrow		= project->read_byte(bank, pData);		pData++;
	road_left.bValid	= (arrow & 0x02) != 0;
	road_right.bValid	= (arrow & 0x01) != 0;
	road_up.bValid		= (arrow & 0x08) != 0;
	road_down.bValid	= (arrow & 0x04) != 0;
	BYTE arrow2		= project->read_byte(bank, pData);		pData++;
	road_left.bParam	= (arrow2 & 0x02) != 0;
	road_right.bParam	= (arrow2 & 0x01) != 0;
	road_up.bParam		= (arrow2 & 0x08) != 0;
	road_down.bParam	= (arrow2 & 0x04) != 0;
	SADRS pText		= project->read_sadrs(bank, pData);		pData += 2;
	level_num		= project->read_word(bank, pData);		pData += 2;
	if (road_right.bValid){
		road_right.area_num	= project->read_word(bank, pData);	pData += 2;
		road_right.road_num	= project->read_word(bank, pData);	pData += 2;
	}
	if (road_left.bValid){
		road_left.area_num	= project->read_word(bank, pData);	pData += 2;
		road_left.road_num	= project->read_word(bank, pData);	pData += 2;
	}
	if (road_down.bValid){
		road_down.area_num	= project->read_word(bank, pData);	pData += 2;
		road_down.road_num	= project->read_word(bank, pData);	pData += 2;
	}
	if (road_up.bValid){
		road_up.area_num	= project->read_word(bank, pData);	pData += 2;
		road_up.road_num	= project->read_word(bank, pData);	pData += 2;
	}
	text.length		= project->read_byte(bank, pText);		pText++;
	text.param		= project->read_byte(bank, pText);		pText++;
	project->read_data(text.string, text.length, bank, pText);
	bValid = true;
	return pData;
}

SADRS AreaDataEx::get_end(const ProjectManager *project, BANK bank, SADRS adrs) const{
	// f[^ޑOɃTCY𓾂
	SADRS pData		= project->read_sadrs(bank, adrs + 1);
	pData += 2;		// pos_x
	pData += 2;		// pos_y
	pData++;		// icon
	pData++;		// arrow
	pData++;		// arrow2
	pData += 2;		// pText
	pData += 2;		// level_num
	if (road_right.bValid)	pData += 4;
	if (road_left.bValid)	pData += 4;
	if (road_down.bValid)	pData += 4;
	if (road_up.bValid)		pData += 4;
	pData++;		// text.length
	pData++;		// text.param
	pData += text.length;	// text.string
	return pData;
}

SADRS AreaDataEx::store(ProjectManager *project, BANK bank, SADRS adrs) const{
	project->write_byte(type, bank, adrs);
	SADRS pData		= project->read_sadrs(bank, adrs + 1);
	project->write_word(pos_x, bank, pData);	pData += 2;
	project->write_word(pos_y, bank, pData);	pData += 2;
	project->write_byte(icon, bank, pData);		pData++;
	BYTE arrow = (BYTE)(road_right.bValid!=false) | ((BYTE)(road_left.bValid!=false)<<1)
		| ((BYTE)(road_down.bValid!=false)<<2) | ((BYTE)(road_up.bValid!=false)<<3);
	BYTE arrow2 = (BYTE)(road_right.bParam!=false) | ((BYTE)(road_left.bParam!=false)<<1)
		| ((BYTE)(road_down.bParam!=false)<<2) | ((BYTE)(road_up.bParam!=false)<<3);
	project->write_byte(arrow, bank, pData);	pData++;
	project->write_byte(arrow2, bank, pData);	pData++;
	SADRS pPtrText = pData;						pData += 2;		// pText
	project->write_word(level_num, bank, pData); pData += 2;
	if (road_right.bValid){
		project->write_word(road_right.area_num, bank, pData);		pData += 2;
		project->write_word(road_right.road_num, bank, pData);		pData += 2;
	}
	if (road_left.bValid){
		project->write_word(road_left.area_num, bank, pData);		pData += 2;
		project->write_word(road_left.road_num, bank, pData);		pData += 2;
	}
	if (road_down.bValid){
		project->write_word(road_down.area_num, bank, pData);		pData += 2;
		project->write_word(road_down.road_num, bank, pData);		pData += 2;
	}
	if (road_up.bValid){
		project->write_word(road_up.area_num, bank, pData);			pData += 2;
		project->write_word(road_up.road_num, bank, pData);			pData += 2;
	}
	project->write_sadrs(pData, bank, pPtrText);	// pText
	project->write_byte(text.length, bank, pData);					pData++;
	project->write_byte(text.param, bank, pData);					pData++;
	project->write_data(text.string, text.length, bank, pData);		pData += text.length;
	return pData;
}




