
///////////////////////////////////////////////////////////////////////////////
//
//	bananas.cpp
//

#include "project.h"
#include "id.h"



BananasManager::BananasManager(ProjectManager *_project) : project(_project){
	banana_set_count = GetPrivateProfileInt("Bananas", "BananaSetCount", 0x6D, RomInfoFileName);
	if (banana_set_count <= 0 || banana_set_count > 0xFF) banana_set_count = 0x6D;
	pBananaSetAdrs	 = GetPrivateProfileInt("Bananas", "BananaSets", 0x3DCA5D, RomInfoFileName);
	banana_set_name	 = new char[banana_set_count][256];
	banana_set_data	 = new BananaSetData[banana_set_count];
	for (int i=0; i<banana_set_count; i++){
		char s[16];
		sprintf_s(s, sizeof s, "%02X", i);
		GetPrivateProfileString("BananaSetName", s, "?", banana_set_name[i], 256, RomInfoFileName);
	}
}

BananasManager::~BananasManager(){
	delete[] banana_set_data;
	delete[] banana_set_name;
}


void BananasManager::load(){
	for (int i=0; i<banana_set_count; i++){
		SADRS adrs_bnset = pBananaSetAdrs.adrs + project->read_sadrs(pBananaSetAdrs.bank, pBananaSetAdrs.adrs + i*2);
		banana_set_data[i].load(project, pBananaSetAdrs.bank, adrs_bnset);
	}
}


int BananasManager::get_banana_set_count(){
	return banana_set_count;
}

const char *BananasManager::get_banana_set_name(int banana_set_num){
	return banana_set_name[banana_set_num];
}

BananaSetData *BananasManager::get_banana_set_data(int banana_set_num){
	return &banana_set_data[banana_set_num];
}



//=================================================================================================


void BananaSetData::load(const ProjectManager *project, BANK bank, SADRS adrs){
	unsigned char *pdata = (unsigned char *)project->get_pointer(bank, adrs);
	width  = *pdata++;
	height = *pdata++;
	int x=0, y=height-1, bc=0;
	while (x<width){
		if (x<BNSET_WIDTH && y<BNSET_HEIGHT){
			set[x][y] = ( ((*pdata) & (1<<bc)) != 0 );
		}
		if (++bc == 8){
			bc = 0;
			pdata++;
		}
		if (y){
			y--;
		}else{
			y = height-1;
			x++;
		}
	}
}







