
///////////////////////////////////////////////////////////////////////////////
//
//	image.h
//

#ifndef _IMAGE_
#define _IMAGE_

///////////////////////////////////////////////////////////////////////////////

#include <windows.h>
#include "base.h"
#include "window.h"

class ProjectManager;
class ImageManager;



struct StaticImageHeader8{
	BYTE big_chip_count;
	BYTE small_chip_count;
	BYTE small_chip_offset;
	BYTE third_chip_count;
	BYTE third_chip_offset;
	BYTE big2_chip_offset;
	BYTE b6;
	BYTE big2_chip_count2;
public:
	void load(ProjectManager *project, LADRS adrs);
	int get_all_chip_count();
	int get_pattern_big0(int big_chip_num);
	int get_pattern_big1(int big_chip_num);
	int get_pattern_big2(int big_chip_num);
	int get_pattern_big3(int big_chip_num);
	int get_pattern_small(int small_chip_num);
	int get_pattern_third(int third_chip_num);
};



//	DynamicImageIterator: Aj[Vf[^̓ǂݏ
//---------------------------------------------------------
class DynamicImageIterator{
private:
	ProjectManager *project;
	BANK bank;
	SADRS adrs;
public:
	DynamicImageIterator(ProjectManager *_proj, BANK _bank, SADRS _adrs);
	DynamicImageIterator(ImageManager *_image, SADRS _adrs);
	DynamicImageIterator(const DynamicImageIterator &_src);
	~DynamicImageIterator();
	SADRS get_adrs() const;
	BYTE get_command() const;
	WORD get_value() const;
	WORD get_value2() const;
	void set_command(BYTE command) const;
	void set_value(WORD value) const;
	void set_value2(WORD value2) const;
	DynamicImageIterator operator ++(int dummy);
	DynamicImageIterator &operator ++();
	operator bool();
	bool operator ==(const DynamicImageIterator &r) const;
	bool operator !=(const DynamicImageIterator &r) const;
};



//	ImageManager: Î~ƃAj[VǗ
//---------------------------------------------------------
class ImageManager : public RomData_Base{

private:
	int static_count;			// Î~̐(W3432)
	LADRS pStaticImageAdrs;		// Î~f[^̃AhX(W0x3C8000)
	LADRS pStaticActionAdrs;	// Î~ANṼAhX(W0x3CB600)
	int dynamic_count;			// Aj[V(L`XvCg)̐(W780)
	LADRS pDynamicImageAdrs;	// Aj[Vf[^̃AhX(W0x390000)
	int palette_count;			// XvCgppbg̐(W234)
	LADRS pPaletteAdrs;			// XvCgppbgAhX(W0x3D5FEE)
public:
	int get_static_count();
	BANK get_static_action_bank();
	LADRS get_static_image(int static_num);
	SADRS get_static_action(int static_num);
	int get_dynamic_count();
	BANK get_dynamic_bank();
	SADRS get_dynamic_image(int dynamic_num);
	WORD get_dynamic_param(int dynamic_num);
	DynamicImageIterator get_dynamic(int dynamic_num);
	int get_palette_count();
	BANK get_palette_bank();
	SADRS get_palette_data(int palette_num);
	WORD *get_palette(int palette_num);

private:
	ProjectManager *project;
public:
	ProjectManager *get_project(){ return project; }


public:
	ImageManager(ProjectManager *_project);
	~ImageManager();


};



///////////////////////////////////////////////////////////////////////////////

#endif // _IMAGE_


