
///////////////////////////////////////////////////////////////////////////////
//
//	isd.cpp
//

#include "isd.h"
#include "project.h"
#include "id.h"


///////////////////////////////////////////////////////////////////////////////


void LevelISD::load_data(const ProjectManager *project, BANK bank, SADRS adrs){
	palette1		= project->read_byte(bank, adrs + 0x00);
	effect			= project->read_byte(bank, adrs + 0x02);
	music			= project->read_byte(bank, adrs + 0x04);
	palette2		= project->read_word(bank, adrs + 0x06);
	pointer1		= project->read_word(bank, adrs + 0x08);
	pointer2		= project->read_word(bank, adrs + 0x0A);
	fg_scroll		= project->read_byte(bank, adrs + 0x0C);
	tilemap_graph	= project->read_byte(bank, adrs + 0x0D);
	redraw			= project->read_byte(bank, adrs + 0x0E);
	tilemap_scroll	= project->read_byte(bank, adrs + 0x10);
	map_num			= project->read_byte(bank, adrs + 0x12);
	view_thg		= project->read_byte(bank, adrs + 0x13);
	special_action	= project->read_byte(bank, adrs + 0x14);
}

void LevelISD::store_data(ProjectManager *project, BANK bank, SADRS adrs){
	project->write_byte(palette1,		bank, adrs + 0x00);
	project->write_byte(effect,			bank, adrs + 0x02);
	project->write_byte(music,			bank, adrs + 0x04);
	project->write_word(palette2,		bank, adrs + 0x06);
	project->write_word(pointer1,		bank, adrs + 0x08);
	project->write_word(pointer2,		bank, adrs + 0x0A);
	project->write_byte(fg_scroll,		bank, adrs + 0x0C);
	project->write_byte(tilemap_graph,	bank, adrs + 0x0D);
	project->write_byte(redraw,			bank, adrs + 0x0E);
	project->write_byte(tilemap_scroll,	bank, adrs + 0x10);
	project->write_byte(map_num,		bank, adrs + 0x12);
	project->write_byte(view_thg,		bank, adrs + 0x13);
	project->write_byte(special_action,	bank, adrs + 0x14);
}



///////////////////////////////////////////////////////////////////////////////



ISDManager::ISDManager(ProjectManager *_project)
: project(_project), style_bank(0x00), style_begin(0x0000), style_count(0), style_name(NULL), data(NULL)
{
	style_bank  = GetPrivateProfileInt("Styles", "Bank", 0x3D, RomInfoFileName);
	style_begin = GetPrivateProfileInt("Styles", "Begin", 0x479E, RomInfoFileName);
	style_count = GetPrivateProfileInt("Styles", "Count", 52, RomInfoFileName);
	if (style_count <= 0 || style_count > 256) style_count = 52;
	style_name = new char[style_count][256];
	data = new LevelISD[style_count];
	WORD style = style_begin;
	char str[8];
	for (int i=0; i<style_count; i++){
		sprintf_s(str, sizeof str, "%04X", style);
		GetPrivateProfileString("StyleName", str, "?", style_name[i], 256, RomInfoFileName);
		style += LevelISD::SIZE;
	}
}

ISDManager::~ISDManager(){
	if (style_name) delete[] style_name;
	if (data) delete[] data;
}

void ISDManager::load(){
	WORD style = style_begin;
	for (int i=0; i<style_count; i++){
		data[i].load_data(project, style_bank, (SADRS)style);
		style += LevelISD::SIZE;
	}
}

void ISDManager::save(){
	WORD style = style_begin;
	for (int i=0; i<style_count; i++){
		data[i].store_data(project, style_bank, (SADRS)style);
		style += LevelISD::SIZE;
	}
}


const char *ISDManager::get_style_name(WORD style){
	int i = (style - style_begin) / LevelISD::SIZE;
	if (i<0 || i>=style_count) return "?";
	return style_name[i];
}

int ISDManager::get_style_count(){
	return style_count;
}

WORD ISDManager::get_style_begin(){
	return style_begin;
}

BANK ISDManager::get_style_bank(){
	return style_bank;
}


const LevelISD *ISDManager::get_style_data(WORD style){
	int i = (style - style_begin) / LevelISD::SIZE;
	return &data[i];
}



BOOL ISDManager::command(WPARAM wParam, LPARAM lParam){
	switch (LOWORD(wParam)){
		case IDM_LEVEL_ISDEDIT:
			{
				LevelSettingData setting;
				project->levels.get_setting_data(&setting);
				if (style_edit_dialog(project->get_hWnd(), setting.style) == IDOK){
					project->levels.redraw_all();
				}
			}
			return TRUE;
	}
	return FALSE;
}

void ISDManager::set_state(BOOL bEnabled){	
	EnableMenuItem(GetMenu(project->get_hWnd()), IDM_LEVEL_ISDEDIT, (bEnabled ? MF_ENABLED : MF_GRAYED) | MF_BYCOMMAND);
}



class ISDSelectDialog : public MyDialog, public RomData_Base{
	ISDManager *isd;
	LPCSTR get_template() const{ return "ISD_SELECTDLG"; }
	BOOL Proc(HWND hDlg, UINT msg, WPARAM wParam, LPARAM lParam){
		switch (msg){
			case WM_INITDIALOG:
				init((WORD)lParam);
				return TRUE;
			case WM_COMMAND:
				switch (LOWORD(wParam)){
					case IDC_LIST0:
						if (HIWORD(wParam) == LBN_SELCHANGE){
							sel_changed(SendDlgItemMessage(hWnd, IDC_LIST0, LB_GETCURSEL, 0, 0));
							return TRUE;
						}else if (HIWORD(wParam) == LBN_DBLCLK){
							PostMessage(hWnd, WM_COMMAND, IDOK, 0);
						}
						return FALSE;
					case IDOK:
						{
							int isel = (int)SendDlgItemMessage(hWnd, IDC_LIST0, LB_GETCURSEL, 0, 0);
							WORD style_begin = isd->get_style_begin();
							EndDialog(hDlg, (INT_PTR)(style_begin + isel*21));
						}
						return TRUE;
					case IDCANCEL:
						EndDialog(hDlg, (INT_PTR)INVALID_STYLE);
						return TRUE;
				}
				return FALSE;
		}
		return FALSE;
	}
	void init(WORD style_init){
		HWND hwStyleList = GetDlgItem(hWnd, IDC_LIST0);
		char s[256];
		int style_count = isd->get_style_count();
		WORD style_begin = isd->get_style_begin();
		WORD style = style_begin;
		for (int i=0; i<style_count; i++){
			sprintf_s(s, sizeof s, "%04X: %s", style, isd->style_name[i]);
			SendMessage(hwStyleList, LB_INSERTSTRING, i, (LPARAM)s);
			style += 21;
		}
		int isel_init = (style_init - style_begin) / 21;
		if (isel_init >= 0 && isel_init < style_count){
			SendMessage(hwStyleList, LB_SETCURSEL, isel_init, 0);
			sel_changed(isel_init);
		}
	}
	void sel_changed(int isel){
		WORD style = isd->get_style_begin() + isel*21;
		char s[32];
		const LevelISD &data = isd->data[isel];
		sprintf_s(s, sizeof s, "%02X", data.palette1);
		SetDlgItemText(hWnd, IDC_EDIT0, s);
		sprintf_s(s, sizeof s, "%02X", data.effect);
		SetDlgItemText(hWnd, IDC_EDIT1, s);
		sprintf_s(s, sizeof s, "%02X", data.music);
		SetDlgItemText(hWnd, IDC_EDIT2, s);
		sprintf_s(s, sizeof s, "%04X", data.palette2);
		SetDlgItemText(hWnd, IDC_EDIT3, s);
		sprintf_s(s, sizeof s, "%04X", data.pointer1);
		SetDlgItemText(hWnd, IDC_EDIT5, s);
		sprintf_s(s, sizeof s, "%04X", data.pointer2);
		SetDlgItemText(hWnd, IDC_EDIT6, s);
		sprintf_s(s, sizeof s, "%02X", data.fg_scroll);
		SetDlgItemText(hWnd, IDC_EDIT7, s);
		sprintf_s(s, sizeof s, "%02X", data.tilemap_graph);
		SetDlgItemText(hWnd, IDC_EDIT8, s);
		sprintf_s(s, sizeof s, "%02X", data.redraw);
		SetDlgItemText(hWnd, IDC_EDIT9, s);
		sprintf_s(s, sizeof s, "%02X", data.tilemap_scroll);
		SetDlgItemText(hWnd, IDC_EDITA, s);
		sprintf_s(s, sizeof s, "%02X", data.map_num);
		SetDlgItemText(hWnd, IDC_EDITB, s);
		sprintf_s(s, sizeof s, "%02X", data.view_thg);
		SetDlgItemText(hWnd, IDC_EDITC, s);
		sprintf_s(s, sizeof s, "%02X", data.special_action);
		SetDlgItemText(hWnd, IDC_EDITD, s);
		CheckDlgButton(hWnd, IDC_CHECK0, (data.special_action & 0x01) != 0);
		CheckDlgButton(hWnd, IDC_CHECK1, (data.special_action & 0x02) != 0);
		CheckDlgButton(hWnd, IDC_CHECK2, (data.special_action & 0x04) != 0);
		CheckDlgButton(hWnd, IDC_CHECK3, (data.special_action & 0x08) != 0);
		CheckDlgButton(hWnd, IDC_CHECK4, (data.special_action & 0x10) != 0);
		CheckDlgButton(hWnd, IDC_CHECK5, (data.special_action & 0x20) != 0);
		CheckDlgButton(hWnd, IDC_CHECK6, (data.special_action & 0x40) != 0);
		CheckDlgButton(hWnd, IDC_CHECK7, (data.special_action & 0x80) != 0);
	}
public:
	ISDSelectDialog(ISDManager *_isd) : isd(_isd){}
	~ISDSelectDialog(){}
};


WORD ISDManager::style_select_dialog(HWND hw_owner, WORD init_style){
	ISDSelectDialog dialog(this);
	return dialog.dialog_box(hw_owner, (LPARAM)init_style);
}



