
///////////////////////////////////////////////////////////////////////////////
//
//	level_bananas.cpp
//

#include <windows.h>
#include <stdio.h>
#include "project.h"
#include "id.h"



void BananaAllocData::load_data(const ProjectManager *project, BANK bank, SADRS adrs, WORD code){
	DWORD data;
	project->read_data(&data, sizeof data, bank, adrs);
	type = (data & 0xFF000000) >> 24;
	switch (code){
		case 0x0000:
			pdx = ((data & 0x00007FF0) >> 4) * 8;
			pdy = ((data & 0x00FF8000) >> 15) * 8;
			break;
		case 0x0004:
			pdx = ((data & 0x00000FF0) >> 4) * 8;
			pdy = ((data & 0x00FFF000) >> 12) * 8;
			break;
		case 0x0005:
			pdx = ((data & 0x000007F0) >> 4) * 8;
			pdy = ((data & 0x00FFF800) >> 11) * 8;
			break;
		case 0x0006: case 0x0007:
			pdx = ((data & 0x00001FF0) >> 4) * 8;
			pdy = ((data & 0x00FFE000) >> 13) * 8;
			break;
		case 0x0008:
			pdx = ((data & 0x0000FFF0) >> 4) * 8;
			pdy = ((data & 0x00FF0000) >> 16) * 8;
			break;
		default:
			return;
	}
	return;
}

void BananaAllocData::normalize(WORD code){
	switch (code){
		case 0x0000:
			pdx &= 0x3FF8;
			pdy &= 0x0FF8;
			break;
		case 0x0004:
			pdx &= 0x07F8;
			pdy &= 0x7FF8;
			break;
		case 0x005:
			pdx &= 0x03F8;
			pdy &= 0xFFF8;
			break;
		case 0x0006: case 0x0007:
			pdx &= 0x0FF8;
			pdy &= 0x3FF8;
			break;
		case 0x0008:
			pdx &= 0x7FF8;
			pdy &= 0x07F8;
			break;
	}
	return;
}

void BananaAllocData::store_data(ProjectManager *project, BANK bank, SADRS adrs, WORD code) const{
	DWORD data = type << 24;
	switch (code){
		case 0x0000:
			data |= ((pdx/8) & 0x07FF) << 4;
			data |= ((pdy/8) & 0x01FF) << 15;
			break;
		case 0x0004:
			data |= ((pdx/8) & 0x00FF) << 4;
			data |= ((pdy/8) & 0x0FFF) << 12;
			break;
		case 0x0005:
			data |= ((pdx/8) & 0x007F) << 4;
			data |= ((pdy/8) & 0x1FFF) << 11;
			break;
		case 0x0006: case 0x0007:
			data |= ((pdx/8) & 0x01FF) << 4;
			data |= ((pdy/8) & 0x07FF) << 13;
			break;
		case 0x0008:
			data |= ((pdx/8) & 0x0FFF) << 4;
			data |= ((pdy/8) & 0x00FF) << 16;
			break;
		default:
			return;
	}
	project->write_data(&data, 4, bank, adrs);
	return;
}


//-----------------------------------------------------------------------------


void LevelBananasData::load_data(const ProjectManager *project, BANK bank, SADRS adrs, int level_num, WORD code){
	WORD adrs_this = project->read_word(bank, adrs + level_num*2);
	WORD adrs_next = project->read_word(bank, adrs + (level_num+1)*2);
	int size = (adrs_next - adrs_this) / BananaAllocData::size;
	SADRS p = adrs + adrs_this;
	BananaAllocData banana;

	bananas.clear();
	for (int i=0; i<size; i++){
		banana.load_data(project, bank, p, code);
		bananas.push_back(banana);
		p+=BananaAllocData::size;
	}
	return;
}

void LevelBananasData::store_data(ProjectManager *project, BANK bank, SADRS adrs, int level_num, WORD code) const{
	WORD adrs_this = project->read_word(bank, adrs + level_num*2);
	WORD adrs_next = project->read_word(bank, adrs + (level_num+1)*2);
	int size = (adrs_next - adrs_this) / BananaAllocData::size;
	SADRS p = adrs + adrs_this;

	std::list<BananaAllocData>::const_iterator i, i_end = bananas.end();
	int n;
	for (i=bananas.begin(), n=0; i!=i_end && n<size; i++, n++){
		(*i).store_data(project, bank, p, code);
		p+=BananaAllocData::size;
	}
	return;
}

void LevelBananasData::clear(){
	bananas.clear();
	return;
}







