
///////////////////////////////////////////////////////////////////////////////
//
//	level_camum.cpp
//

#include <stdio.h>
#include "id.h"
#include "project.h"



//=================================================================================================



LevelCameraApply_UndoMethod::LevelCameraApply_UndoMethod(
	ProjectManager *_project_manager, LevelManager *_level_manager,
	LevelCamerasManager *_cameras_manager, int _level_num, int _camera_num,
	const CameraAllocData *_data_before, const CameraAllocData *_data_after)
	: project_manager(_project_manager), level_manager(_level_manager), cameras_manager(_cameras_manager),
	level_num(_level_num), camera_num(_camera_num), data_before(*_data_before), data_after(*_data_after)
{
}

LevelCameraApply_UndoMethod::~LevelCameraApply_UndoMethod(){}


void LevelCameraApply_UndoMethod::undo(){
	if (level_manager->get_level_num() == level_num){
		method_in_level(true);
	}else{
		method_out_of_level(true);
	}
	return;
}

void LevelCameraApply_UndoMethod::redo(){
	if (level_manager->get_level_num() == level_num){
		method_in_level(false);
	}else{
		method_out_of_level(false);
	}
	return;
}

void LevelCameraApply_UndoMethod::method_in_level(bool bUndo){
	char s[128], s2[128];
	level_manager->select_page(LEVEL_CAMERAS);
	level_manager->excluse_pages(LEVEL_CAMERAS);
	LevelCamerasData::iterator i = cameras_manager->get_iterator(camera_num);
	*i = (bUndo ? data_before : data_after);
	SADRS adrs_levels = GetPrivateProfileInt("Cameras", "Levels", 0x9323, RomInfoFileName);
	SADRS adrs_cameras = adrs_levels + project_manager->read_word(0x3D, adrs_levels + level_num*2);
	(*i).store_data(project_manager, 0x3D,
		adrs_cameras + camera_num*CameraAllocData::size, level_manager->get_code());
	HWND hwList = GetDlgItem(cameras_manager->hWnd, IDC_LIST0);
	SendMessage(hwList, WM_SETREDRAW, (WPARAM)FALSE, 0);
	SendMessage(hwList, LB_DELETESTRING, (WPARAM)camera_num, 0);
	cameras_manager->get_camera_text(s, sizeof s, &*i);
	sprintf_s(s2, sizeof s2, "#%d: %s", camera_num, s);
	SendMessage(hwList, LB_INSERTSTRING, (WPARAM)camera_num, (LPARAM)s2);
	SendMessage(hwList, WM_SETREDRAW, (WPARAM)TRUE, 0);
	SendMessage(hwList, LB_SETCURSEL, (WPARAM)camera_num, 0);
	cameras_manager->output_camera_data(&*i, camera_num);
	cameras_manager->focus_on_camera(camera_num);
	return;
}

void LevelCameraApply_UndoMethod::method_out_of_level(bool bUndo){
	level_manager->load_level(level_num);
	method_in_level(bUndo);
	return;
}



//=================================================================================================

LevelCameraUpDown_UndoMethod::LevelCameraUpDown_UndoMethod(
	ProjectManager *_project_manager, LevelManager *_level_manager,
	LevelCamerasManager *_cameras_manager, int _level_num, int _camera_former)
	: project_manager(_project_manager), level_manager(_level_manager), cameras_manager(_cameras_manager),
	level_num(_level_num), camera_former(_camera_former), camera_latter(_camera_former+1)
{
}

LevelCameraUpDown_UndoMethod::~LevelCameraUpDown_UndoMethod(){}

void LevelCameraUpDown_UndoMethod::method(bool bUp){
	if (level_manager->get_level_num() == level_num){
		method_in_level(bUp);
	}else{
		method_out_of_level(bUp);
	}
	return;
}

void LevelCameraUpDown_UndoMethod::method_in_level(bool bUp){
	char s[256], s2[256];
	level_manager->select_page(LEVEL_CAMERAS);
	level_manager->excluse_pages(LEVEL_CAMERAS);
	LevelCamerasData::iterator i_former = cameras_manager->get_iterator(camera_former);
	LevelCamerasData::iterator i_latter = cameras_manager->get_iterator(camera_latter);
	HWND hwList = GetDlgItem(cameras_manager->get_hWnd(), IDC_LIST0);
	SendMessage(hwList, WM_SETREDRAW, (WPARAM)FALSE, 0);
	SendMessage(hwList, LB_DELETESTRING, (WPARAM)camera_latter, 0);
	SendMessage(hwList, LB_DELETESTRING, (WPARAM)camera_former, 0);
	CameraAllocData data_temp = *i_former;
	*i_former = *i_latter;
	*i_latter = data_temp;
	cameras_manager->get_camera_text(s2, sizeof s2, &*i_former);
	sprintf_s(s, sizeof s, "#%d: %s", camera_former, s2);
	SendMessage(hwList, LB_INSERTSTRING, (WPARAM)camera_former, (LPARAM)s);
	cameras_manager->get_camera_text(s2, sizeof s2, &*i_latter);
	sprintf_s(s, sizeof s, "#%d: %s", camera_latter, s2);
	SendMessage(hwList, LB_INSERTSTRING, (WPARAM)camera_latter, (LPARAM)s);
	SendMessage(hwList, WM_SETREDRAW, (WPARAM)TRUE, 0);
	int camera_num = bUp ? camera_former : camera_latter;
	SendMessage(hwList, LB_SETCURSEL, (WPARAM)camera_num, 0);
	cameras_manager->output_camera_data(&*(bUp ? i_former : i_latter), camera_num);
	// ROMɔf
	SADRS adrs_levels = GetPrivateProfileInt("Cameras", "Levels", 0x9323, RomInfoFileName);
	SADRS adrs_cameras = adrs_levels + project_manager->read_word(0x3D, adrs_levels + level_num*2);
	SADRS adrs_former = adrs_cameras + camera_former*CameraAllocData::size;
	SADRS adrs_latter = adrs_cameras + camera_latter*CameraAllocData::size;
	(*i_former).store_data(project_manager, 0x3D, adrs_former, level_manager->get_code());
	(*i_latter).store_data(project_manager, 0x3D, adrs_latter, level_manager->get_code());
	cameras_manager->focus_on_camera(camera_num);
	return;
}

void LevelCameraUpDown_UndoMethod::method_out_of_level(bool bUp){
	level_manager->load_level(level_num);
	method_in_level(bUp);
	return;
}

//-------------------------------------------------------------------------------------------------

LevelCameraUp_UndoMethod::LevelCameraUp_UndoMethod(
	ProjectManager *_project_manager, LevelManager *_level_manager,
	LevelCamerasManager *_cameras_manager, int _level_num, int _camera_num)
	: LevelCameraUpDown_UndoMethod(_project_manager, _level_manager, _cameras_manager,
	_level_num, _camera_num-1)
{
}

LevelCameraUp_UndoMethod::~LevelCameraUp_UndoMethod(){}

void LevelCameraUp_UndoMethod::undo(){
	method(false);
	return;
}

void LevelCameraUp_UndoMethod::redo(){
	method(true);
	return;
}

//-------------------------------------------------------------------------------------------------

LevelCameraDown_UndoMethod::LevelCameraDown_UndoMethod(
	ProjectManager *_project_manager, LevelManager *_level_manager,
	LevelCamerasManager *_cameras_manager, int _level_num, int _camera_num)
	: LevelCameraUpDown_UndoMethod(_project_manager, _level_manager, _cameras_manager,
	_level_num, _camera_num)
{
}

LevelCameraDown_UndoMethod::~LevelCameraDown_UndoMethod(){}

void LevelCameraDown_UndoMethod::undo(){
	method(true);
}

void LevelCameraDown_UndoMethod::redo(){
	method(false);
}

//=================================================================================================


LevelCameraSort_UndoMethod::LevelCameraSort_UndoMethod(
	ProjectManager *_project_manager, LevelManager *_level_manager,
	LevelCamerasManager *_cameras_manager, int _level_num, int _camera_before, int _camera_after,
	const LevelCamerasData &_data_before, const LevelCamerasData &_data_after)
	: project_manager(_project_manager), level_manager(_level_manager), cameras_manager(_cameras_manager),
	level_num(_level_num), camera_before(_camera_before), camera_after(_camera_after),
	data_before(_data_before), data_after(_data_after)
{
}

LevelCameraSort_UndoMethod::~LevelCameraSort_UndoMethod(){}


void LevelCameraSort_UndoMethod::undo(){
	if (level_manager->get_level_num() == level_num){
		method_in_level(true);
	}else{
		method_out_of_level(true);
	}
	return;
}

void LevelCameraSort_UndoMethod::redo(){
	if (level_manager->get_level_num() == level_num){
		method_in_level(false);
	}else{
		method_out_of_level(false);
	}
	return;
}

void LevelCameraSort_UndoMethod::method_in_level(bool bUndo){
	int n;
	char s[256], s2[256];
	level_manager->select_page(LEVEL_CAMERAS);
	level_manager->excluse_pages(LEVEL_CAMERAS);
	LevelCamerasData::iterator i, i_end;
	HWND hwList = GetDlgItem(cameras_manager->get_hWnd(), IDC_LIST0);
	SendMessage(hwList, WM_SETREDRAW, (WPARAM)FALSE, 0);
	for (i=cameras_manager->data.begin(), i_end=cameras_manager->data.end(); i!=i_end; i++){
		SendMessage(hwList, LB_DELETESTRING, (WPARAM)0, 0);
	}
	cameras_manager->data = bUndo ? data_before : data_after;
	for (i=cameras_manager->data.begin(), i_end=cameras_manager->data.end(), n=0; i!=i_end; i++, n++){
		cameras_manager->get_camera_text(s2, sizeof s2, &(*i));
		sprintf_s(s, sizeof s, "#%d: %s", n, s2);
		SendMessage(hwList, LB_INSERTSTRING, (WPARAM)n, (LPARAM)s);
	}
	SendMessage(hwList, WM_SETREDRAW, (WPARAM)TRUE, 0);
	int camera_num = bUndo ? camera_before : camera_after;
	SendMessage(hwList, LB_SETCURSEL, (WPARAM)camera_num, 0);
	cameras_manager->sel_changed(camera_num);
	// ROMɔf
	SADRS adrs_levels = GetPrivateProfileInt("Cameras", "Levels", 0x9323, RomInfoFileName);
	cameras_manager->data.store_data(project_manager, 0x3D, adrs_levels, level_num, level_manager->get_code());
	cameras_manager->focus_on_camera(camera_num);
	return;
}

void LevelCameraSort_UndoMethod::method_out_of_level(bool bUndo){
	level_manager->load_level(level_num);
	method_in_level(bUndo);
	return;
}




