
///////////////////////////////////////////////////////////////////////////////
//
//	level_setting.h
//

#ifndef _LEVEL_SETTING_
#define _LEVEL_SETTING_

///////////////////////////////////////////////////////////////////////////////

#include <windows.h>
#include "base.h"
#include "window.h"


class ProjectManager;
class ISDManager;


//---------------------------------------------------------
struct LevelSettingData{

	static const int START_MAX	= 64;
	static const int JUMP_MAX	= 64;
	static const int UPARAM_MAX	= 64;

	WORD type;
	BYTE bonus_type;
	SADRS style;
	BYTE map_index;
	BYTE param1;
	BYTE param2;
	BYTE param3;
	BYTE param4;
	BYTE param5;
	BYTE param6;
	int start_count;
	struct{
		BYTE param;
		int pdx;
		int pdy;
	} starts[START_MAX];
	int jump_count;
	WORD jumps[JUMP_MAX];
	int uparam_count;
	struct{
		WORD pl;
		WORD ph;
	} uparams[UPARAM_MAX];

	int get_size() const;
	int load_data(const ProjectManager *project, BANK bank, SADRS adrs);
	int store_data(ProjectManager *project, BANK bank, SADRS adrs, int size_max) const;

};



//---------------------------------------------------------
class LevelSettingDialog : public MyDialog, public RomData_Base{

	ISDManager *isd;
	int level_num;
	LevelSettingData data;
	int max_size;
	virtual LPCSTR get_template() const;
	virtual BOOL Proc(HWND hWnd, UINT msg, WPARAM wParam, LPARAM lParam);
	void init_dialog();
	void level_type_changed(WORD new_value);
	void start_selchanged(int start_num);
	void jump_selchanged(int jump_num);
	void uparam_selchanged(int uparam_num);
	void get_text_start(char *dst, int nSize, int index, BYTE param, int pdx, int pdy);
	void get_text_jump(char *dst, int nSize, int index, WORD jump);
	void get_text_uparam(char *dst, int nSize, int index, WORD pl, WORD ph);
	bool start_active;
	bool jump_active;
	bool uparam_active;
	void apply_start();
	void apply_jump();
	void apply_uparam();
	void select_level();
	void add_start();
	void add_jump();
	void add_uparam();
	void delete_start();
	void delete_jump();
	void delete_uparam();
	bool end_setting();
	void select_style();

public:

	LevelSettingDialog(const LevelSettingData *data_src, int _max_size, int _level_num, ISDManager *_isd);
	~LevelSettingDialog();
	void get_data(LevelSettingData *data_dst);

};





///////////////////////////////////////////////////////////////////////////////

#endif // _LEVEL_SETTING_





