
///////////////////////////////////////////////////////////////////////////////
//
//	level_sprites.h
//

#ifndef _LEVEL_SPRITES_
#define _LEVEL_SPRITES_

///////////////////////////////////////////////////////////////////////////////

#include <windows.h>
#include <list>
#include <map>
#include "base.h"


class LevelSpritesManager;
class LevelManager;
class ProjectManager;


//	XvCgzũf[^(\Ă)
//---------------------------------------------------------
struct SpriteAllocData{
	static const int size = 8;
	int pdx;
	int pdy;
	WORD type;
	WORD param;
	void load_data(const ProjectManager *project, BANK bank, SADRS adrs);
	void store_data(ProjectManager *project, BANK bank, SADRS adrs) const;
};


//	Xe[WƂ̃XvCgzuf[^
//---------------------------------------------------------
class LevelSpritesData : public RomData_Base{

	std::list<SpriteAllocData> sprites;

public:

	typedef std::list<SpriteAllocData>::iterator iterator;
	typedef std::list<SpriteAllocData>::reverse_iterator reverse_iterator;

	int get_size() const;
	int load_data(const ProjectManager *project, BANK bank, SADRS adrs);
	int store_data(ProjectManager *project, BANK bank, SADRS adrs, int size_max) const;
	void clear();
	iterator begin(){ return sprites.begin(); }
	iterator end(){ return sprites.end(); }
	reverse_iterator rbegin(){ return sprites.rbegin(); }
	reverse_iterator rend(){ return sprites.rend(); }
	size_t get_sprite_count(){ return sprites.size(); }
	iterator erase(iterator i){ return sprites.erase(i); }
	iterator insert(iterator i, const SpriteAllocData &data){ return sprites.insert(i, data); }
	SpriteAllocData &front(){ return sprites.front(); }
	SpriteAllocData &back(){ return sprites.back(); }

};



//	XvCǵuKpvɊւAhDEhD
//---------------------------------------------------------
class LevelSpriteApply_UndoMethod : public UndoMethod{
	ProjectManager *project_manager;
	LevelManager *level_manager;
	LevelSpritesManager *sprites_manager;
	SpriteAllocData data_before;
	SpriteAllocData data_after;
	int level_num;
	int sprite_num;
	virtual LPCSTR text_undo() const{ return "Move Sprites"; }
	virtual LPCSTR text_redo() const{ return "Move Sprites"; }
	virtual void undo();
	virtual void redo();
	void method_in_level(bool bUndo);
	void method_out_of_level(bool bUndo);
public:
	LevelSpriteApply_UndoMethod(ProjectManager *_project_manager, LevelManager *_level_manager,
		LevelSpritesManager *_sprites_manager, int _level_num, int _sprite_num,
		const SpriteAllocData *_data_before, const SpriteAllocData *_data_after);
	~LevelSpriteApply_UndoMethod();
};

//	XvCǵuǉvu폜vAhDEhDNX
//---------------------------------------------------------
class LevelSpriteAddDelete_UndoMethod : public UndoMethod{
	void method_in_level(bool bAdd);
	void method_out_of_level(bool bAdd);
protected:
	ProjectManager *project_manager;
	LevelManager *level_manager;
	LevelSpritesManager *sprites_manager;
	SpriteAllocData data;
	int level_num;
	int sprite_num;
	LevelSpriteAddDelete_UndoMethod(ProjectManager *_project_manager, LevelManager *_level_manager,
		LevelSpritesManager *_sprites_manager, int _level_num, int _sprite_num,
		const SpriteAllocData *_data);
	~LevelSpriteAddDelete_UndoMethod();
	void do_add();
	void do_delete();
};

//	XvCǵuǉvAhDEhD
//---------------------------------------------------------
class LevelSpriteAdd_UndoMethod : public LevelSpriteAddDelete_UndoMethod{
	virtual LPCSTR text_undo() const{ return "Add Sprite"; }
	virtual LPCSTR text_redo() const{ return "Add Sprite"; }
	virtual void undo();
	virtual void redo();
public:
	LevelSpriteAdd_UndoMethod(ProjectManager *_project_manager, LevelManager *_level_manager,
		LevelSpritesManager *_sprites_manager, int _level_num, int _sprite_num, const SpriteAllocData *_data);
	~LevelSpriteAdd_UndoMethod();
};

//	XvCǵu폜vAhDEhD
//---------------------------------------------------------
class LevelSpriteDelete_UndoMethod : public LevelSpriteAddDelete_UndoMethod{
	virtual LPCSTR text_undo() const{ return "Delete Sprite"; }
	virtual LPCSTR text_redo() const{ return "Delete Sprite"; }
	virtual void undo();
	virtual void redo();
public:
	LevelSpriteDelete_UndoMethod(ProjectManager *_project_manager, LevelManager *_level_manager,
		LevelSpritesManager *_sprites_manager, int _level_num, int _sprite_num, const SpriteAllocData *_data);
	~LevelSpriteDelete_UndoMethod();
};

//	XvCg̏㉺ړɊւAhDEhDNX
//---------------------------------------------------------
class LevelSpriteUpDown_UndoMethod : public UndoMethod{
	void method_in_level(bool bUp);
	void method_out_of_level(bool bUp);
protected:
	ProjectManager *project_manager;
	LevelManager *level_manager;
	LevelSpritesManager *sprites_manager;
	int level_num;
	int sprite_former;
	int sprite_latter;
	LevelSpriteUpDown_UndoMethod(ProjectManager *_project_manager, LevelManager *_level_manager,
		LevelSpritesManager *_sprites_manager, int _level_num, int _sprite_former);
	~LevelSpriteUpDown_UndoMethod();
	void do_up();
	void do_down();
};

//	XvCguցvAhDEhD
//---------------------------------------------------------
class LevelSpriteUp_UndoMethod : public LevelSpriteUpDown_UndoMethod{
	virtual LPCSTR text_undo() const{ return "Sprite up"; }
	virtual LPCSTR text_redo() const{ return "Sprite up"; }
	virtual void undo();
	virtual void redo();
public:
	LevelSpriteUp_UndoMethod(ProjectManager *_project_manager, LevelManager *_level_manager,
		LevelSpritesManager *_sprites_manager, int _level_num, int _sprite_num);
	~LevelSpriteUp_UndoMethod();
};

//	XvCguցvAhDEhD
//---------------------------------------------------------
class LevelSpriteDown_UndoMethod : public LevelSpriteUpDown_UndoMethod{
	virtual LPCSTR text_undo() const{ return "Sprite down"; }
	virtual LPCSTR text_redo() const{ return "Sprite down"; }
	virtual void undo();
	virtual void redo();
public:
	LevelSpriteDown_UndoMethod(ProjectManager *_project_manager, LevelManager *_level_manager,
		LevelSpritesManager *_sprites_manager, int _level_num, int _sprite_num);
	~LevelSpriteDown_UndoMethod();
};

//	XvCǵu\[gvAhDEhD
//---------------------------------------------------------
class LevelSpriteSort_UndoMethod : public UndoMethod{
	ProjectManager *project_manager;
	LevelManager *level_manager;
	LevelSpritesManager *sprites_manager;
	int level_num;
	LevelSpritesData data_before;
	LevelSpritesData data_after;
	int sprite_before;
	int sprite_after;
	virtual LPCSTR text_undo() const{ return "Sort Sprites"; }
	virtual LPCSTR text_redo() const{ return "Sort Sprites"; }
	virtual void undo();
	virtual void redo();
	void method_in_level(bool bUndo);
	void method_out_of_level(bool bUndo);
public:
	LevelSpriteSort_UndoMethod(ProjectManager *_project_manager, LevelManager *_level_manager,
		LevelSpritesManager *_sprites_manager, int _level_num, int _sprite_before, int _sprite_after,
		const LevelSpritesData &_data_before, const LevelSpritesData &_data_after);
	~LevelSpriteSort_UndoMethod();
};



//	XvCgzu̕ҏWiAXvCgҏW_CAOǗ
//---------------------------------------------------------
class LevelSpritesManager : public RomData_Base, public MyDialog, public TabPage{

	friend class LevelSpriteApply_UndoMethod;
	friend class LevelSpriteAddDelete_UndoMethod;
	friend class LevelSpriteUpDown_UndoMethod;
	friend class LevelSpriteSort_UndoMethod;

private:

	virtual LPCSTR get_text() const{ return "Sprites"; }
	virtual LPCSTR get_template() const{ return "LEVEL_SPRITES"; }
	virtual BOOL Proc(HWND hDlg, UINT msg, WPARAM wParam, LPARAM lParam);

	inline static LONG MakeSpriteLONG(WORD dynamic_num, WORD palette_num, bool h_flip, bool v_flip){
		return (LONG)dynamic_num | (((LONG)(palette_num&0x3FFF))<<16) | (((LONG)h_flip)<<30) | (((LONG)v_flip)<<31);
	}
	inline static WORD GetDynamicNum(LONG sprite_long){	return (WORD)sprite_long;					}
	inline static WORD GetPaletteNum(LONG sprite_long){	return (((DWORD)sprite_long)>>16) & 0x3FFF;	}
	inline static bool GetHFlip(LONG sprite_long){	return (sprite_long & 0x40000000) != 0;	}
	inline static bool GetVFlip(LONG sprite_long){	return (sprite_long & 0x80000000) != 0;	}
	struct SpriteImage{
		int index;
		RECT rtClip;
	};
	typedef std::map<LONG, SpriteImage> SpriteImageMap;
	typedef std::pair<LONG, SpriteImage> SpriteImageElement;
	SpriteImageMap sprite_image_map;		// 摜ʎq(LONG)->C[WXgf[^(SpriteImage)̑Ή
	HIMAGELIST hilSpriteImage;
	typedef std::map<int, LONG> SpriteTypeMap;
	typedef std::pair<int, LONG> SpriteTypeElement;
	SpriteTypeMap sprite_type_map;			// XvCg̎(int)->摜ʎq(LONG)̑Ή
	static const int SPRITE_WIDTH = 32;
	static const int SPRITE_HEIGHT = 36;
	HPEN	hpnSpriteLine;
	HPEN	hpnSpriteSel;
	HBRUSH	hbrSpriteRect;
	HBRUSH	hbrSpriteLine;
	HBRUSH	hbrSpriteSel;
	HFONT	hftSpriteText;
	enum ShowSprites{
		SPR_HIDE=0,
		SPR_NORMAL,
		SPR_SIMPLE
	} iShowSprites;
	int isel;
	bool bDragging;
	int dragging_x;
	int dragging_y;
	int drag_temp_x;
	int drag_temp_y;
	bool unable_edit_change;

	int level_num;
	LevelSpritesData data;
	int max_size;

	LevelManager *level;
	ProjectManager *project;

	void prepare_sprite_image(WORD sprite_type);
	void draw_mini_chip(BYTE buf[256][256][3], int pos_x, int pos_y, const BYTE *data, const WORD *palette, bool h_flip, bool v_flip);
	void set_menuitem_state();
	void sel_changed(int sel);
	void input_sprite_data(SpriteAllocData *sprite);
	void output_sprite_data(const SpriteAllocData *sprite, int iIndex);
	void jump(int sel);
	void focus_on_sprite(int sel);
	void spr_apply(int sel);
	void spr_add(int sel);
	void spr_delete(int sel);
	void spr_up(int sel);
	void spr_down(int sel);
	void spr_sort(int sel);
	void type_changed();
	void select_sprite();
	void get_sprite_text(char *dst, size_t size, const SpriteAllocData *sprite);
	void get_sprite_name(char *dst, size_t size, WORD type);
	LevelSpritesData::iterator get_iterator(int iIndex);

public:

	LevelSpritesManager(LevelManager *_level, ProjectManager *_project);
	~LevelSpritesManager();
	virtual BOOL create(HWND hwTab);
	BOOL create_sys();
	void save_data();
	void load(int _level_num);
	void clear();
	void draw(BitmapBuffer *buf, const RECT *rtRedraw);
	void draw_f(BitmapBuffer *buf, const RECT *rtRedraw);
	BOOL command(WPARAM wParam, LPARAM lParam);
	BOOL translate_dialog_message(MSG *msg);
	void set_state(bool bEnabled);
	bool mouseover(int pdx, int pdy);
	bool accepts_click(int pdx, int pdy);
	void dragging(int pdx, int pdy);
	void release_click(bool bApply);
	void release_sel();
	void select_sprite(int num);

};





///////////////////////////////////////////////////////////////////////////////

#endif // _LEVEL_SPRITES_






