
///////////////////////////////////////////////////////////////////////////////
//
//	level_tilechp.cpp
//

#include "id.h"
#include "project.h"



bool LevelTilemapManager::load_chipdata(const LevelISD *style, const MapData *map_data)
{
	extern int ExpandData(unsigned char *buf, unsigned int buf_size, const unsigned char *buf_src);
	int exp_size;

	// ~j`bv̔zuf[^
	WORD (*mini_chips)[4][4];				// mini_chips[chip_num][y][x]; x,y=0,1,2,3
	mini_chips = new WORD[16 * 64][4][4];
	exp_size = ExpandData((unsigned char *)mini_chips, 16*64*4*4*2,
		(const unsigned char *)project->get_pointer(map_data->chip_method_adrs));
	if (exp_size == -1){
		delete[] mini_chips;
		MessageBox(project->get_hWnd(), "Failed to deploy configuration data mapping chip.", "Error", MB_OK | MB_ICONEXCLAMATION);
		return false;
	}
	int chip_count = exp_size / 32;

	// ~j`bṽOtBbNf[^
	int graph_sys_num = style->tilemap_graph;
	LADRS lGraphSysAdrs = (DWORD)GetPrivateProfileInt("Graphics", "GraphSysAdrs", 0x3D819A, RomInfoFileName);
	WORD graph_sys_offset = project->read_word(lGraphSysAdrs.bank, lGraphSysAdrs.adrs + graph_sys_num*2);
	SADRS graph_ptr = lGraphSysAdrs.adrs + graph_sys_offset;
	LADRS lGraphics = (DWORD)0;
	LADRS lGraphicsSecond = (DWORD)0;
	bool bGraphicsCompressed;
	WORD non_comp_size = 0;
	WORD non_comp_size_second = 0;
	while (true){
		BYTE first		= project->read_byte(lGraphSysAdrs.bank, graph_ptr);	graph_ptr++;
		WORD second		= project->read_word(lGraphSysAdrs.bank, graph_ptr);	graph_ptr += 2;
		WORD command	= project->read_word(lGraphSysAdrs.bank, graph_ptr);	graph_ptr += 2;
		WORD param		= project->read_word(lGraphSysAdrs.bank, graph_ptr);	graph_ptr += 2;
		if (first == 0x00){
			break;
		}
		if (command == 0xA000){
			lGraphics.bank = first - 0xC0;
			lGraphics.adrs = second;
			bGraphicsCompressed = true;
		}else if (command == 0x2000){
			lGraphics.bank = first - 0xC0;
			lGraphics.adrs = second;
			bGraphicsCompressed = false;
			non_comp_size = param;
		}else if (command == 0x4420){
			lGraphicsSecond.bank = first - 0xC0;
			lGraphicsSecond.adrs = second;
			non_comp_size_second = param;
		}
	}
	if (lGraphics.get() == 0){
		delete[] mini_chips;
		MessageBox(project->get_hWnd(), "Map Chip Configuration Data not found",
			"Error", MB_OK | MB_ICONEXCLAMATION);
		return false;
	}

	int mini_chip_count;
	BYTE *graphics = new BYTE[65536];
	if (bGraphicsCompressed){
		exp_size = ExpandData((unsigned char*)graphics, 65536, (const unsigned char*)project->get_pointer(lGraphics));
		if (exp_size == -1){
			delete[] graphics;
			delete[] mini_chips;
			MessageBox(project->get_hWnd(), "Failed to load Map Chip Configuration Data",
				"Error", MB_OK | MB_ICONEXCLAMATION);
			return false;
		}
		mini_chip_count = exp_size / 32;
	}else{
		project->read_data(graphics, non_comp_size, lGraphics);
		if (non_comp_size < 0x4840){
			ZeroMemory(graphics + non_comp_size, 0x4840 - non_comp_size);
		}
		if (lGraphicsSecond.get() != 0){
			project->read_data(graphics + 0x4840, non_comp_size_second, lGraphicsSecond);
		}
		mini_chip_count = (0x4840 + non_comp_size_second) / 32;
	}
	BANK bank = project->maps.get_map_bank();

	// DIB쐬
	struct{
		BITMAPINFOHEADER header;
		RGBQUAD palette[256];
	} bmp_info;

	DWORD width = 512;
	DWORD height = (chip_count%16 ? (chip_count/16+1) : chip_count/16) * 32;
	DWORD bmp_size = width * height;

	bmp_info.header.biSize			= sizeof(BITMAPINFOHEADER);
	bmp_info.header.biWidth			= width;
	bmp_info.header.biHeight		= height;
	bmp_info.header.biPlanes		= 1;
	bmp_info.header.biBitCount		= 8;
	bmp_info.header.biCompression	= BI_RGB;
	bmp_info.header.biSizeImage		= bmp_size;
	bmp_info.header.biXPelsPerMeter	= 0;
	bmp_info.header.biYPelsPerMeter	= 0;
	bmp_info.header.biClrUsed		= 0;
	bmp_info.header.biClrImportant	= 0;

	// pbǵAISDŐݒ肵̂΂AȂ΃}bvɐݒ肳Ă̂gpB
	WORD wChipPalettes		= style->palette2;
	if (wChipPalettes == 0){
		wChipPalettes	= map_data->tilemap_color;
	}
	// ISD̃pbg1lApbgǂݍ݁B
	switch (style->palette1){
		case 0x00:
			load_whole_palette(bmp_info.palette, project->isd.get_style_bank(), wChipPalettes);
			load_partial_palette(bmp_info.palette, project->isd.get_style_bank(), 0x1710, 0, 32*4);
			break;
		case 0x01:
			load_whole_palette(bmp_info.palette, project->isd.get_style_bank(), wChipPalettes);
			load_partial_palette(bmp_info.palette, project->isd.get_style_bank(), 0x268E, 16, 4*4);
			break;
		case 0x02:
			load_whole_palette(bmp_info.palette, project->isd.get_style_bank(), 0x2AEE);
			break;
		case 0x03:
			load_whole_palette(bmp_info.palette, project->isd.get_style_bank(), 0x29EE);
			break;
		case 0x04:
			load_whole_palette(bmp_info.palette, project->isd.get_style_bank(), wChipPalettes);
			load_partial_palette(bmp_info.palette, project->isd.get_style_bank(), 0x324E, 0, 32*4);
			break;
		case 0x05:
			load_whole_palette(bmp_info.palette, project->isd.get_style_bank(), wChipPalettes);
			load_partial_palette(bmp_info.palette, project->isd.get_style_bank(), 0x304E, 0, 32*4);
			break;
		case 0x06:
			load_whole_palette(bmp_info.palette, project->isd.get_style_bank(), wChipPalettes);
			load_partial_palette(bmp_info.palette, project->isd.get_style_bank(), 0x2EEE, 0, 32*4);
			break;
		case 0x07:
			load_whole_palette(bmp_info.palette, project->isd.get_style_bank(), wChipPalettes);
			load_partial_palette(bmp_info.palette, project->isd.get_style_bank(), 0x15F0, 12, 4*4);
			break;
		case 0x08:
			load_whole_palette(bmp_info.palette, project->isd.get_style_bank(), wChipPalettes);
			load_partial_palette(bmp_info.palette, project->isd.get_style_bank(), 0x0DD0, 0, 4*4);
			break;
		case 0x09:
			load_whole_palette(bmp_info.palette, project->isd.get_style_bank(), wChipPalettes);
			break;
		case 0x0A:
			load_whole_palette(bmp_info.palette, project->isd.get_style_bank(), wChipPalettes);
			load_partial_palette(bmp_info.palette, project->isd.get_style_bank(), 0x326E, 16, 4*4);	// ۂ͏tc
			break;
		case 0x0B:
			load_whole_palette(bmp_info.palette, project->isd.get_style_bank(), wChipPalettes);
			break;
		case 0x0C:
			load_whole_palette(bmp_info.palette, project->isd.get_style_bank(), wChipPalettes);
			load_partial_palette(bmp_info.palette, project->isd.get_style_bank(), 0x2BEE, 0, 32*4);
			break;
		case 0x0D:
			load_whole_palette(bmp_info.palette, project->isd.get_style_bank(), wChipPalettes);
			load_partial_palette(bmp_info.palette, project->isd.get_style_bank(), 0x07F0, 0, 32*4);
			break;
		case 0x0E:
			load_whole_palette(bmp_info.palette, project->isd.get_style_bank(), wChipPalettes);
			load_partial_palette(bmp_info.palette, project->isd.get_style_bank(), 0x268E, 112, 4*4);
			break;
		case 0x0F:
			load_whole_palette(bmp_info.palette, project->isd.get_style_bank(), wChipPalettes);
			break;
		case 0x10:
			load_whole_palette(bmp_info.palette, project->isd.get_style_bank(), wChipPalettes);
			load_partial_palette(bmp_info.palette, project->isd.get_style_bank(), 0x3A4E, 0, 32*4);
			break;
		case 0x11:
			load_whole_palette(bmp_info.palette, project->isd.get_style_bank(), wChipPalettes);
			break;
		case 0x12:
			load_whole_palette(bmp_info.palette, project->isd.get_style_bank(), wChipPalettes);
			load_partial_palette(bmp_info.palette, project->isd.get_style_bank(), 0x2DCE, 0, 4*4);
			break;
		case 0x13:
			load_whole_palette(bmp_info.palette, project->isd.get_style_bank(), wChipPalettes);
			break;
		case 0x14:
			load_whole_palette(bmp_info.palette, project->isd.get_style_bank(), 0x1610);
			break;
		default:
			load_whole_palette(bmp_info.palette, project->isd.get_style_bank(), wChipPalettes);
	}

	// n`ŎgpȂ(F128ȍ~)FœhԂ
	for (int i=0; i<8; i++){
		bmp_info.palette[i*16].rgbRed	= 16;
		bmp_info.palette[i*16].rgbGreen	= 16;
		bmp_info.palette[i*16].rgbBlue	= 64;
	}
	for (int i=128; i<256; i++){
		bmp_info.palette[i].rgbRed		= 16;
		bmp_info.palette[i].rgbGreen	= 16;
		bmp_info.palette[i].rgbBlue		= 64;
	}

	BYTE *data = (LPBYTE)HeapAlloc(GetProcessHeap(), HEAP_ZERO_MEMORY, bmp_size);
	//BYTE *data = new BYTE[bmp_size];


	for (int chip_index=0; chip_index<chip_count; chip_index++){
		for (int j=0; j<4; j++){
			for (int i=0; i<4; i++){
				WORD tile_data = mini_chips[chip_index][j][i];
				WORD mini_chip_num	= (tile_data & 0x03FF);
				BYTE palette_num	= (tile_data & 0x1C00) >> 10;
				BYTE palette_ofs	= palette_num * 16;
				bool hflip			= (tile_data & 0x4000) != 0;
				bool vflip			= (tile_data & 0x8000) != 0;
				if (mini_chip_num >= mini_chip_count) mini_chip_num = 0;
				if (hflip){
					if (vflip){
						for (int iy=0; iy<8; iy++){
							int data_p = (height-1-((chip_index/16)*32 + j*8 + iy))*width + (chip_index%16)*32 + i*8;
							BYTE p0 = graphics[mini_chip_num*32 + (7-iy)*2];
							BYTE p1 = graphics[mini_chip_num*32 + (7-iy)*2 + 1];
							BYTE p2 = graphics[mini_chip_num*32 + (7-iy)*2 + 16];
							BYTE p3 = graphics[mini_chip_num*32 + (7-iy)*2 + 17];
							data[data_p + 7] = (((p0 & 0x80)>>7) | ((p1 & 0x80)>>6) | ((p2 & 0x80)>>5) | ((p3 & 0x80)>>4)) + palette_ofs;
							data[data_p + 6] = (((p0 & 0x40)>>6) | ((p1 & 0x40)>>5) | ((p2 & 0x40)>>4) | ((p3 & 0x40)>>3)) + palette_ofs;
							data[data_p + 5] = (((p0 & 0x20)>>5) | ((p1 & 0x20)>>4) | ((p2 & 0x20)>>3) | ((p3 & 0x20)>>2)) + palette_ofs;
							data[data_p + 4] = (((p0 & 0x10)>>4) | ((p1 & 0x10)>>3) | ((p2 & 0x10)>>2) | ((p3 & 0x10)>>1)) + palette_ofs;
							data[data_p + 3] = (((p0 & 0x08)>>3) | ((p1 & 0x08)>>2) | ((p2 & 0x08)>>1) | ((p3 & 0x08)   )) + palette_ofs;
							data[data_p + 2] = (((p0 & 0x04)>>2) | ((p1 & 0x04)>>1) | ((p2 & 0x04)   ) | ((p3 & 0x04)<<1)) + palette_ofs;
							data[data_p + 1] = (((p0 & 0x02)>>1) | ((p1 & 0x02)   ) | ((p2 & 0x02)<<1) | ((p3 & 0x02)<<2)) + palette_ofs;
							data[data_p + 0] = (((p0 & 0x01)   ) | ((p1 & 0x01)<<1) | ((p2 & 0x01)<<2) | ((p3 & 0x01)<<3)) + palette_ofs;
						}
					}else{
						for (int iy=0; iy<8; iy++){
							int data_p = (height-1-((chip_index/16)*32 + j*8 + iy))*width + (chip_index%16)*32 + i*8;
							BYTE p0 = graphics[mini_chip_num*32 + iy*2];
							BYTE p1 = graphics[mini_chip_num*32 + iy*2 + 1];
							BYTE p2 = graphics[mini_chip_num*32 + iy*2 + 16];
							BYTE p3 = graphics[mini_chip_num*32 + iy*2 + 17];
							data[data_p + 7] = (((p0 & 0x80)>>7) | ((p1 & 0x80)>>6) | ((p2 & 0x80)>>5) | ((p3 & 0x80)>>4)) + palette_ofs;
							data[data_p + 6] = (((p0 & 0x40)>>6) | ((p1 & 0x40)>>5) | ((p2 & 0x40)>>4) | ((p3 & 0x40)>>3)) + palette_ofs;
							data[data_p + 5] = (((p0 & 0x20)>>5) | ((p1 & 0x20)>>4) | ((p2 & 0x20)>>3) | ((p3 & 0x20)>>2)) + palette_ofs;
							data[data_p + 4] = (((p0 & 0x10)>>4) | ((p1 & 0x10)>>3) | ((p2 & 0x10)>>2) | ((p3 & 0x10)>>1)) + palette_ofs;
							data[data_p + 3] = (((p0 & 0x08)>>3) | ((p1 & 0x08)>>2) | ((p2 & 0x08)>>1) | ((p3 & 0x08)   )) + palette_ofs;
							data[data_p + 2] = (((p0 & 0x04)>>2) | ((p1 & 0x04)>>1) | ((p2 & 0x04)   ) | ((p3 & 0x04)<<1)) + palette_ofs;
							data[data_p + 1] = (((p0 & 0x02)>>1) | ((p1 & 0x02)   ) | ((p2 & 0x02)<<1) | ((p3 & 0x02)<<2)) + palette_ofs;
							data[data_p + 0] = (((p0 & 0x01)   ) | ((p1 & 0x01)<<1) | ((p2 & 0x01)<<2) | ((p3 & 0x01)<<3)) + palette_ofs;
						}
					}
				}else{
					if (vflip){
						for (int iy=0; iy<8; iy++){
							int data_p = (height-1-((chip_index/16)*32 + j*8 + iy))*width + (chip_index%16)*32 + i*8;
							BYTE p0 = graphics[mini_chip_num*32 + (7-iy)*2];
							BYTE p1 = graphics[mini_chip_num*32 + (7-iy)*2 + 1];
							BYTE p2 = graphics[mini_chip_num*32 + (7-iy)*2 + 16];
							BYTE p3 = graphics[mini_chip_num*32 + (7-iy)*2 + 17];
							data[data_p + 0] = (((p0 & 0x80)>>7) | ((p1 & 0x80)>>6) | ((p2 & 0x80)>>5) | ((p3 & 0x80)>>4)) + palette_ofs;
							data[data_p + 1] = (((p0 & 0x40)>>6) | ((p1 & 0x40)>>5) | ((p2 & 0x40)>>4) | ((p3 & 0x40)>>3)) + palette_ofs;
							data[data_p + 2] = (((p0 & 0x20)>>5) | ((p1 & 0x20)>>4) | ((p2 & 0x20)>>3) | ((p3 & 0x20)>>2)) + palette_ofs;
							data[data_p + 3] = (((p0 & 0x10)>>4) | ((p1 & 0x10)>>3) | ((p2 & 0x10)>>2) | ((p3 & 0x10)>>1)) + palette_ofs;
							data[data_p + 4] = (((p0 & 0x08)>>3) | ((p1 & 0x08)>>2) | ((p2 & 0x08)>>1) | ((p3 & 0x08)   )) + palette_ofs;
							data[data_p + 5] = (((p0 & 0x04)>>2) | ((p1 & 0x04)>>1) | ((p2 & 0x04)   ) | ((p3 & 0x04)<<1)) + palette_ofs;
							data[data_p + 6] = (((p0 & 0x02)>>1) | ((p1 & 0x02)   ) | ((p2 & 0x02)<<1) | ((p3 & 0x02)<<2)) + palette_ofs;
							data[data_p + 7] = (((p0 & 0x01)   ) | ((p1 & 0x01)<<1) | ((p2 & 0x01)<<2) | ((p3 & 0x01)<<3)) + palette_ofs;
						}
					}else{
						for (int iy=0; iy<8; iy++){
							int data_p = (height-1-((chip_index/16)*32 + j*8 + iy))*width + (chip_index%16)*32 + i*8;
							BYTE p0 = graphics[mini_chip_num*32 + iy*2];
							BYTE p1 = graphics[mini_chip_num*32 + iy*2 + 1];
							BYTE p2 = graphics[mini_chip_num*32 + iy*2 + 16];
							BYTE p3 = graphics[mini_chip_num*32 + iy*2 + 17];
							data[data_p + 0] = (((p0 & 0x80)>>7) | ((p1 & 0x80)>>6) | ((p2 & 0x80)>>5) | ((p3 & 0x80)>>4)) + palette_ofs;
							data[data_p + 1] = (((p0 & 0x40)>>6) | ((p1 & 0x40)>>5) | ((p2 & 0x40)>>4) | ((p3 & 0x40)>>3)) + palette_ofs;
							data[data_p + 2] = (((p0 & 0x20)>>5) | ((p1 & 0x20)>>4) | ((p2 & 0x20)>>3) | ((p3 & 0x20)>>2)) + palette_ofs;
							data[data_p + 3] = (((p0 & 0x10)>>4) | ((p1 & 0x10)>>3) | ((p2 & 0x10)>>2) | ((p3 & 0x10)>>1)) + palette_ofs;
							data[data_p + 4] = (((p0 & 0x08)>>3) | ((p1 & 0x08)>>2) | ((p2 & 0x08)>>1) | ((p3 & 0x08)   )) + palette_ofs;
							data[data_p + 5] = (((p0 & 0x04)>>2) | ((p1 & 0x04)>>1) | ((p2 & 0x04)   ) | ((p3 & 0x04)<<1)) + palette_ofs;
							data[data_p + 6] = (((p0 & 0x02)>>1) | ((p1 & 0x02)   ) | ((p2 & 0x02)<<1) | ((p3 & 0x02)<<2)) + palette_ofs;
							data[data_p + 7] = (((p0 & 0x01)   ) | ((p1 & 0x01)<<1) | ((p2 & 0x01)<<2) | ((p3 & 0x01)<<3)) + palette_ofs;
						}
					}
				}
			}
		}
	}

	// 256FDIB}bv`bvobt@ɓ]
	struct LogPalette{
		WORD palVersion;
		WORD palNumEntries;
		PALETTEENTRY palEntry[128];
	} log_pal;			
	log_pal.palVersion		= 0x0300;
	log_pal.palNumEntries	= 128;
	for (int i=0; i<128; i++){
		log_pal.palEntry[i].peRed	= bmp_info.palette[i].rgbRed;
		log_pal.palEntry[i].peGreen	= bmp_info.palette[i].rgbGreen;
		log_pal.palEntry[i].peBlue	= bmp_info.palette[i].rgbBlue;
		log_pal.palEntry[i].peFlags	= NULL;
	}
	HPALETTE hPalette = CreatePalette((LPLOGPALETTE)&log_pal);
	for (int i=0; i<128; i++){
		*((WORD*)bmp_info.palette + i) = (WORD)i;
	}
	map_chip.create(GetDC(level->get_hWnd()), width*4, height);
	SelectPalette(map_chip.get_HDC(), hPalette, TRUE);
	RealizePalette(map_chip.get_HDC());
	int bounded_chip_count = ((chip_count + 0x0F) >> 4) << 4;
	for (int chip_index=0; chip_index<bounded_chip_count; chip_index++){
		int x = (chip_index % 16) * 32;
		int y = (chip_index / 16) * 32;
		int y2 = height - 32 - y;
		StretchDIBits(map_chip.get_HDC(), x, y, 32, 32, x, y2, 32, 32,
			(LPCVOID)data, (LPBITMAPINFO)&bmp_info, DIB_PAL_COLORS, SRCCOPY);
		StretchDIBits(map_chip.get_HDC(), x + 512 + 31, y, -32, 32, x, y2, 32, 32,
			(LPCVOID)data, (LPBITMAPINFO)&bmp_info, DIB_PAL_COLORS, SRCCOPY);
		StretchDIBits(map_chip.get_HDC(), x + 1024, y + 31, 32, -32, x, y2, 32, 32,
			(LPCVOID)data, (LPBITMAPINFO)&bmp_info, DIB_PAL_COLORS, SRCCOPY);
		StretchDIBits(map_chip.get_HDC(), x + 1536 + 31, y + 31, -32, -32, x, y2, 32, 32,
			(LPCVOID)data, (LPBITMAPINFO)&bmp_info, DIB_PAL_COLORS, SRCCOPY);
	}
	DeleteObject(hPalette);
	HeapFree(GetProcessHeap(), 0, (LPVOID)data);
	//delete[] data;
	delete[] graphics;
	delete[] mini_chips;
	return true;
}



void LevelTilemapManager::load_whole_palette(RGBQUAD palette[], BANK bank, SADRS adrs){
	for (int i=0; i<128; i++){
		WORD palcolor = project->read_word(bank, adrs + i*2);
		palette[i].rgbRed	= ((palcolor & 0x001F)) * 8;
		palette[i].rgbGreen	= ((palcolor & 0x03E0) >> 5) * 8;
		palette[i].rgbBlue	= ((palcolor & 0x7C00) >> 10) * 8;
		palette[i].rgbReserved	= 0;
	}
	return;
}

void LevelTilemapManager::load_partial_palette(RGBQUAD palette[], BANK bank, SADRS adrs, int iStart, int iCount){
	for (int i=0; i<iCount; i++){
		WORD palcolor = project->read_word(bank, adrs + i*2);
		palette[i + iStart].rgbRed		= ((palcolor & 0x001F)) * 8;
		palette[i + iStart].rgbGreen	= ((palcolor & 0x03E0) >> 5) * 8;
		palette[i + iStart].rgbBlue		= ((palcolor & 0x7C00) >> 10) * 8;
		palette[i + iStart].rgbReserved	= 0;
	}
	return;
}








