
///////////////////////////////////////////////////////////////////////////////
//
//	level_tilemap.cpp
//

#include "id.h"
#include "project.h"


LevelTilemapManager::LevelTilemapManager(LevelManager *_level, ProjectManager *_project)
: level(_level), project(_project)
{
	tilemap = NULL;
	tilemap_org = NULL;
	actionmap = NULL;
}

LevelTilemapManager::~LevelTilemapManager(){
	if (tilemap) delete[] tilemap;
	if (tilemap_org) delete[] tilemap_org;
	if (actionmap) delete[] actionmap;
	DeleteObject(hpnNormal);
	DeleteObject(hpnType1);
	DeleteObject(hpnType2);
	DeleteObject(hpnType3);
	DeleteObject(hbrNormal);
	DeleteObject(hbrType1);
	DeleteObject(hbrType2);
	DeleteObject(hbrType3);
}

BOOL LevelTilemapManager::create(){
	// yEuV̍쐬
	hpnNormal = CreatePen(PS_SOLID, 1, RGB(128, 128, 128));
	hpnType1  = CreatePen(PS_SOLID, 1, RGB(128, 0, 0));
	hpnType2  = CreatePen(PS_SOLID, 1, RGB(128, 64, 0));
	hpnType3  = CreatePen(PS_SOLID, 1, RGB(64, 64, 64));
	hbrNormal = CreateSolidBrush(RGB(128, 128, 128));
	hbrType1  = CreateSolidBrush(RGB(128, 0, 0));
	hbrType2  = CreateSolidBrush(RGB(128, 64, 0));
	hbrType3  = CreateSolidBrush(RGB(64, 64, 64));
	// ANV`bvHDC, rbg}bv̍쐬
	action_chip.create(GetDC(level->get_hWnd()), 16*32*4, 64*32);
	iShowTilemap = (ShowTilemapType)GetPrivateProfileInt("MapView", "ShowTilemap", TRUE, ConfigFileName);
	set_menuitem_state();
	return TRUE;
}

void LevelTilemapManager::save_data(){
	char s[16];
	sprintf_s(s, sizeof s, "%d", iShowTilemap);
	WritePrivateProfileString("MapView", "ShowTilemap", s, ConfigFileName);
	return;
}

void LevelTilemapManager::set_menuitem_state(){
	MENUITEMINFO mi;
	mi.cbSize	= sizeof mi;
	mi.fMask	= MIIM_STATE;
	mi.fState	= iShowTilemap==MAP_HIDE ? MFS_CHECKED : MFS_UNCHECKED;
	SetMenuItemInfo(GetMenu(project->get_hWnd()), IDM_VIEW_MAP_HIDE, FALSE, &mi);
	mi.fState	= iShowTilemap==MAP_TILEMAP ? MFS_CHECKED : MFS_UNCHECKED;
	SetMenuItemInfo(GetMenu(project->get_hWnd()), IDM_VIEW_MAP_TILEMAP, FALSE, &mi);
	mi.fState	= iShowTilemap==MAP_ACTION ? MFS_CHECKED : MFS_UNCHECKED;
	SetMenuItemInfo(GetMenu(project->get_hWnd()), IDM_VIEW_MAP_ACTION, FALSE, &mi);
	return;
}


void LevelTilemapManager::load(const LevelSettingData *setting, HDC hdc_wnd){
	const LevelISD *style = project->isd.get_style_data(setting->style);	// X^C
	const MapData *map_data = project->maps.get_map_data(style->map_num);	// }bvf[^
	if (setting->type == 0x0004 || setting->type == 0x0005){
		map_valid = false;
		map_cx = 0;
		map_cy = 0;
		map_code = 0x0000;
		map_vert = false;
		return;
	}
	// ANV}bv̓ǂݍ
	LADRS lActionmap	= map_data->action_map;
	actionmap_size		= map_data->action_map_size;
	if (lActionmap.get() != 0x000000 && actionmap_size!=0){
		if (actionmap) delete[] actionmap;
		actionmap = new DWORD[actionmap_size];
		project->read_data(actionmap, actionmap_size*4, lActionmap);
		// ANV`bv̍\
		write_action_chips();
	}
	// n`f[^{̂̓ǂݍ
	if (map_data->tilemap_type & 0x0020){
		// n`
		map_valid = true;
		if (!load_tilemap(setting, map_data)){
			map_valid = false;
			return;
		}
		// `bv摜̓ǂݍ
		if (!load_chipdata(style, map_data)){
			map_valid = false;
			return;
		}
	}else{
		// n`Ȃ(D̏)
		map_valid = false;
		map_cx = 8;
		map_cy = 8;
		map_vert = true;
	}
	return;
}


bool LevelTilemapManager::load_tilemap(const LevelSettingData *setting, const MapData *map_data){
	LADRS lTilemap	= map_data->tilemap_adrs;
	SADRS sMapAlloc	= map_data->allocation_adrs;
	WORD wMapCX		= project->read_word(0x35, sMapAlloc);
	WORD wMapCY		= project->read_word(0x35, sMapAlloc + 2);
	WORD map_left	= project->read_word(0x35, sMapAlloc + 4 + setting->map_index*10) / 32;
	WORD map_right	= project->read_word(0x35, sMapAlloc + 6 + setting->map_index*10) / 32;
	WORD map_top	= project->read_word(0x35, sMapAlloc + 8 + setting->map_index*10) / 32;
	WORD map_bottom	= project->read_word(0x35, sMapAlloc + 10 + setting->map_index*10) / 32;
	map_code		= project->read_word(0x35, sMapAlloc + 12 + setting->map_index*10);
	map_cx			= map_right - map_left;
	map_cy			= map_bottom - map_top;
	WORD *buf_map = new WORD[65536];
	int size;
	if (map_data->tilemap_type & 0x0400){
		// k
		extern int ExpandData(unsigned char *buf, unsigned int buf_size, const unsigned char *buf_src);
		size = ExpandData((unsigned char *)buf_map, 65536*2,
			(const unsigned char *)project->get_pointer(lTilemap));
		if (size == -1){
			delete[] buf_map;
			MessageBox(project->get_hWnd(), "Failed to load Map data", "Error", MB_OK | MB_ICONEXCLAMATION);
			map_vert = (wMapCX & 0x0FFF) != 0;
			return false;
		}
	}else{
		// kȂ
		project->read_data(buf_map, 256, lTilemap);
	}
	// }bv̐؂蔲
	WORD tile, h_flip, v_flip;
	if (tilemap) delete[] tilemap;
	tilemap = new WORD[65536];
	if (tilemap_org) delete[] tilemap_org;
	tilemap_org = new WORD[65536];
	int height;
	project->maps.get_tilemap_attribute(map_data, &map_vert, &height);
	if (map_vert){
		// c
		for (int y=0, gy=map_top; gy<map_bottom; y++, gy++){
			for (int x=0, gx=map_left; gx<map_right; x++, gx++){
				tile = buf_map[gx+gy*height];
				h_flip = tile & 0x4000;
				v_flip = tile & 0x8000;
				tilemap_org[x+y*map_cx] = tile;
				tilemap[x+y*map_cx] = (tile&0x000F) | ((h_flip!=0)<<4)
					| ((v_flip!=0)<<5) | ((tile&0x03F0)<<4);
			}
		}
	}else{
		// 
		for (int x=0, gx=map_left; gx<map_right; x++, gx++){
			for (int y=0, gy=map_top; gy<map_bottom; y++, gy++){
				tile = buf_map[gy+gx*height];
				h_flip = tile & 0x4000;
				v_flip = tile & 0x8000;
				tilemap_org[x+y*map_cx] = tile;
				tilemap[x+y*map_cx] = (tile&0x000F) | ((h_flip!=0)<<4)
					| ((v_flip!=0)<<5) | ((tile&0x03F0)<<4);
			}
		}
	}
	delete[] buf_map;
	return true;
}



void LevelTilemapManager::draw(BitmapBuffer *buf, const RECT *rtRedraw){
	//WORD action;
	WORD tile;
	//bool vflip, hflip;
	//bool aflip;
	if (iShowTilemap == MAP_ACTION && !tilemap_org || !actionmap) return;
	if (map_valid && iShowTilemap!=MAP_HIDE){
		int left = rtRedraw->left/32;
		int top = rtRedraw->top/32;
		int right = min(map_cx-1, rtRedraw->right/32+1);
		int bottom = min(map_cy-1, rtRedraw->bottom/32+1);
		int left_b = rtRedraw->left%32;
		int top_b = rtRedraw->top%32;
		for (int x=left, sx=0; x<=right; x++, sx++){
			for (int y=top, sy=0; y<=bottom; y++, sy++){
				if (iShowTilemap == MAP_TILEMAP){
					BitBlt(buf->get_HDC(), sx*32-left_b, sy*32-top_b, 32, 32,
						map_chip.get_HDC(), (tilemap[x+y*map_cx]&0x00FF)*32,
						(tilemap[x+y*map_cx]&0xFF00)/8, SRCCOPY);
				}else if (iShowTilemap == MAP_ACTION){
					tile = tilemap[x + y*map_cx];
					BitBlt(buf->get_HDC(), sx*32-left_b, sy*32-top_b, 32, 32,
						action_chip.get_HDC(), (tile&0x00FF)*32,
						(tile&0xFF00)/8, SRCCOPY);
				}
			}
		}
	}else{
		buf->select_object(GetStockObject(BLACK_BRUSH));
		buf->rectangle(0, 0,
			min(rtRedraw->right-rtRedraw->left, map_cx*32),
			min(rtRedraw->bottom-rtRedraw->top, map_cy*32));
	}
}


BOOL LevelTilemapManager::command(WPARAM wParam, LPARAM lParam){
	if (LOWORD(wParam) == IDM_VIEW_TILEMAP){
		switch (iShowTilemap){
			case MAP_HIDE:		iShowTilemap = MAP_TILEMAP;	break;
			case MAP_TILEMAP:	iShowTilemap = MAP_ACTION;	break;
			case MAP_ACTION:	iShowTilemap = MAP_HIDE;	break;
		}
		set_menuitem_state();
		level->redraw_default();
		return TRUE;
	}
	if (LOWORD(wParam) == IDM_VIEW_TILEMAP_I){
		switch (iShowTilemap){
			case MAP_HIDE:		iShowTilemap = MAP_ACTION;	break;
			case MAP_TILEMAP:	iShowTilemap = MAP_HIDE;	break;
			case MAP_ACTION:	iShowTilemap = MAP_TILEMAP;	break;
		}
		set_menuitem_state();
		level->redraw_default();
		return TRUE;
	}
	if (LOWORD(wParam) == IDM_VIEW_MAP_HIDE){
		iShowTilemap = MAP_HIDE;
		set_menuitem_state();
		level->redraw_default();
		return TRUE;
	}
	if (LOWORD(wParam) == IDM_VIEW_MAP_TILEMAP){
		iShowTilemap = MAP_TILEMAP;
		set_menuitem_state();
		level->redraw_default();
		return TRUE;
	}
	if (LOWORD(wParam) == IDM_VIEW_MAP_ACTION){
		iShowTilemap = MAP_ACTION;
		set_menuitem_state();
		level->redraw_default();
		return TRUE;
	}
	return FALSE;
}






