
///////////////////////////////////////////////////////////////////////////////
//
//	project.cpp
//

#include <windows.h>
#include <commctrl.h>
#include <stdio.h>
#include <shlwapi.h>
#include "id.h"
#include "project.h"

#pragma warning(disable: 4355)


LPCSTR ProjectManager::pAppNameMainWindow = "MainWindow";
LPCSTR ProjectManager::pClassName = "sdk2ed_main";

BOOL ProjectManager::Init(){
	if (!register_class_ex(
		CS_HREDRAW | CS_VREDRAW,
		(HICON)LoadIcon(hInst, MAKEINTRESOURCE(IDI_SDK2ED)),
		(HICON)LoadIcon(hInst, MAKEINTRESOURCE(IDI_SDK2ED)),
		LoadCursor(NULL, IDC_ARROW),
		(HBRUSH)(COLOR_APPWORKSPACE + 1),
		pClassName,
		"SDK2ED_MENU"))
	{
		return FALSE;
	}
	if (!LevelManager::Init()) return FALSE;
	return TRUE;
}

ProjectManager::ProjectManager()
: undo_buffer(), maps(this), isd(this), levels(this), sprites(this), bananas(this), image(this), areas(this), characters(this),
recent_files(this), test_play(this)
{
	bEnabled = FALSE;
	nChangeStep = 0;
	bChangedInrev = false;
	buf = NULL;
	header = new BYTE[ROM_HEADER_SIZE];
	hwStatus = NULL;
}


ProjectManager::~ProjectManager(){
	if (buf != NULL) delete[] buf;
	delete[] header;
}


BOOL ProjectManager::create(LPCSTR cmdline, int nCmdShow){

	int x, y, w, h, m;
	x = (signed)GetPrivateProfileInt(pAppNameMainWindow, "Pos_X", CW_USEDEFAULT, ConfigFileName);
	y = (signed)GetPrivateProfileInt(pAppNameMainWindow, "Pos_Y", CW_USEDEFAULT, ConfigFileName);
	w = (signed)GetPrivateProfileInt(pAppNameMainWindow, "Width", CW_USEDEFAULT, ConfigFileName);
	h = (signed)GetPrivateProfileInt(pAppNameMainWindow, "Height", CW_USEDEFAULT, ConfigFileName);
	m = (signed)GetPrivateProfileInt(pAppNameMainWindow, "Maximized", 0, ConfigFileName);
	if ((x==CW_USEDEFAULT)||(y==CW_USEDEFAULT)||(w==CW_USEDEFAULT)||(h==CW_USEDEFAULT)){
		x = y = w = h = CW_USEDEFAULT;
	}

	bShowTool	= (BOOL)GetPrivateProfileInt(pAppNameMainWindow, "ToolBar", TRUE, ConfigFileName);
	bShowStatus	= (BOOL)GetPrivateProfileInt(pAppNameMainWindow, "StatusBar", TRUE, ConfigFileName);

	if (!create_window_ex(pClassName, "SDK2 Editor",
		WS_OVERLAPPEDWINDOW | WS_CLIPCHILDREN,
		0,
		x, y, w, h,
		NULL))
	{
		return FALSE;
	}

	if (!sprites.create()) return FALSE;
	if (!areas.create()) return FALSE;

	DragAcceptFiles(hWnd, TRUE);
	set_state(false);
	open_cmdline(cmdline);
	ShowWindow(hWnd, m ? SW_SHOWMAXIMIZED : nCmdShow);
	UpdateWindow(hWnd);
	return TRUE;
}

void ProjectManager::save_window_pos(){
	WINDOWPLACEMENT wp;
	GetWindowPlacement(hWnd, &wp);
	char s[16];
	sprintf_s(s, sizeof s, "%d", bShowTool!=FALSE);
	WritePrivateProfileString(pAppNameMainWindow, "ToolBar", s, ConfigFileName);
	sprintf_s(s, sizeof s, "%d", bShowStatus!=FALSE);
	WritePrivateProfileString(pAppNameMainWindow, "StatusBar", s, ConfigFileName);
	sprintf_s(s, sizeof s, "%d", wp.rcNormalPosition.left);
	WritePrivateProfileString(pAppNameMainWindow, "Pos_X", s, ConfigFileName);
	sprintf_s(s, sizeof s, "%d", wp.rcNormalPosition.top);
	WritePrivateProfileString(pAppNameMainWindow, "Pos_Y", s, ConfigFileName);
	sprintf_s(s, sizeof s, "%d", wp.rcNormalPosition.right - wp.rcNormalPosition.left);
	WritePrivateProfileString(pAppNameMainWindow, "Width", s, ConfigFileName);
	sprintf_s(s, sizeof s, "%d", wp.rcNormalPosition.bottom - wp.rcNormalPosition.top);
	WritePrivateProfileString(pAppNameMainWindow, "Height", s, ConfigFileName);
	sprintf_s(s, sizeof s, "%d", wp.showCmd == SW_SHOWMAXIMIZED);
	WritePrivateProfileString(pAppNameMainWindow, "Maximized", s, ConfigFileName);
	return;
}

void ProjectManager::set_state(bool _bEnabled){
	bEnabled = _bEnabled;
	HMENU hmMain = GetMenu(hWnd);
	UINT uState = (bEnabled ? MF_ENABLED : MF_GRAYED) | MF_BYCOMMAND;
	EnableMenuItem(hmMain, IDM_FILE_SAVE,		uState);
	EnableMenuItem(hmMain, IDM_FILE_SAVEAS,		uState);
	EnableMenuItem(hmMain, IDM_FILE_RELOAD,		uState);
	EnableMenuItem(hmMain, IDM_FILE_CLOSE,		uState);
	EnableMenuItem(hmMain, IDM_FILE_TESTPLAY,	uState);
	EnableMenuItem(hmMain, IDM_EDIT_UNDO,		uState);
	EnableMenuItem(hmMain, IDM_EDIT_REDO,		uState);
	SendMessage(hwTool, TB_ENABLEBUTTON, (WPARAM)IDM_FILE_SAVE, (LPARAM)bEnabled);
	SendMessage(hwTool, TB_ENABLEBUTTON, (WPARAM)IDM_FILE_TESTPLAY, (LPARAM)bEnabled);
	SendMessage(hwTool, TB_ENABLEBUTTON, (WPARAM)IDM_EDIT_UNDO, (LPARAM)bEnabled);
	SendMessage(hwTool, TB_ENABLEBUTTON, (WPARAM)IDM_EDIT_REDO, (LPARAM)bEnabled);
	levels.set_state(bEnabled);
	isd.set_state(bEnabled);
	maps.set_state(bEnabled);
	sprites.set_state(bEnabled);
	areas.set_state(bEnabled);
	refresh_caption();
	return;
}

void ProjectManager::level_changed(){
	refresh_caption();
}

void ProjectManager::refresh_caption(){
	char s[512], s2[256];
	if (bEnabled){
		int level_num = levels.get_level_num();
		sprintf_s(s, sizeof s, "%02X", level_num);
		GetPrivateProfileString("LevelName", s, "?", s2, sizeof s2, RomInfoFileName);
		if (nChangeStep){
			sprintf_s(s, sizeof s, "%s *[%02X: %s] - SDK2 Editor", filetitle, level_num, s2);
		}else{
			sprintf_s(s, sizeof s, "%s [%02X: %s] - SDK2 Editor", filetitle, level_num, s2);
		}
	}else{
		strcpy_s(s, sizeof s, "SDK2 Editor");
	}
	this->set_text(s);
	return;
}

LRESULT ProjectManager::Proc(HWND hWnd, UINT msg, WPARAM wParam, LPARAM lParam){
	LPNMHDR nmhdr;
	switch (msg){
		case WM_CREATE:
			if ((hwStatus = CreateWindowEx(0, STATUSCLASSNAME, NULL,
				WS_CHILD | (bShowStatus? WS_VISIBLE: 0) | SBARS_SIZEGRIP | CCS_BOTTOM,
				0, 0, 0, 0, hWnd, (HMENU)IDC_STATUS0, hInst, NULL)
				) == NULL) return -1;
			{
				int aWidth[] = {0, 0, 0, 0};
				SendMessage(hwStatus, SB_SETPARTS, 4, (LPARAM)aWidth);
			}
			if ((hwTool = CreateWindowEx(0, TOOLBARCLASSNAME, NULL,
				WS_CHILD | (bShowTool? WS_VISIBLE: 0) | CCS_TOP | TBSTYLE_TOOLTIPS,
				0, 0, 0, 0, hWnd, (HMENU)IDC_TOOL0, hInst, NULL)
				) == NULL) return -1;
			{
				HMENU hMenu = GetMenu(hWnd);
				MENUITEMINFO mi;
				mi.cbSize	= sizeof mi;
				mi.fMask	= MIIM_STATE;
				mi.fState	= bShowTool? MFS_CHECKED : MFS_UNCHECKED;
				SetMenuItemInfo(hMenu, IDM_VIEW_TOOL, FALSE, &mi);
				mi.fState	= bShowStatus? MFS_CHECKED : MFS_UNCHECKED;
				SetMenuItemInfo(hMenu, IDM_VIEW_STATUS, FALSE, &mi);
			}
			init_toolbar();
			if (!recent_files.create()) return -1;
			if (!undo_buffer.create(this)) return -1;
			if (!levels.create()) return -1;
			return 0;
		case WM_ACTIVATE:
			//if (LOWORD(wParam) == WA_INACTIVE && (HWND)lParam == levels.inspector.get_hWnd()) break;
			levels.inspector.nc_activate(wParam);
			break;
		case WM_SIZE:
			{
				RECT rt;
				int x=0, y=0, width = (SHORT)LOWORD(lParam), height = (SHORT)HIWORD(lParam);
				if (bShowStatus){
					int aWidth[] = {LOWORD(lParam) - 480, LOWORD(lParam) - 240, LOWORD(lParam) - 120, -1};
					SendMessage(hwStatus, SB_SETPARTS, 4, (LPARAM)aWidth);
					SendMessage(hwStatus, WM_SIZE, wParam, lParam);
					GetWindowRect(hwStatus, &rt);
					height -= rt.bottom - rt.top;
				}
				if (bShowTool){
					SendMessage(hwTool, WM_SIZE, wParam, lParam);
					GetWindowRect(hwTool, &rt);
					height -= rt.bottom - rt.top;
					y += rt.bottom - rt.top;
				}
				levels.move(x, y, width, height, TRUE);
			}
			return 0;
		case WM_ENABLE:
			EnableWindow(levels.inspector.get_hWnd(), wParam ? TRUE : FALSE);
			return 0;
		case WM_NOTIFY:
			switch ((nmhdr = (LPNMHDR)lParam)->code){
				case TTN_GETDISPINFO:
					{
						LPNMTTDISPINFO nmtt = (LPNMTTDISPINFO)nmhdr;
						nmtt->hinst		= hInst;
						nmtt->uFlags	= 0;
						nmtt->lParam	= NULL;
						switch (nmtt->hdr.idFrom){
							case IDM_FILE_OPEN:		nmtt->lpszText = MAKEINTRESOURCE(IDS_FILE_OPEN);		break;
							case IDM_FILE_SAVE:		nmtt->lpszText = MAKEINTRESOURCE(IDS_FILE_SAVE);		break;
							case IDM_EDIT_UNDO:		nmtt->lpszText = MAKEINTRESOURCE(IDS_EDIT_UNDO);		break;
							case IDM_EDIT_REDO:		nmtt->lpszText = MAKEINTRESOURCE(IDS_EDIT_REDO);		break;
							case IDM_VIEW_TILEMAP:	nmtt->lpszText = MAKEINTRESOURCE(IDS_VIEW_TILEMAP);		break;
							case IDM_VIEW_SPRITES:	nmtt->lpszText = MAKEINTRESOURCE(IDS_VIEW_SPRITES);		break;
							case IDM_VIEW_BANANAS:	nmtt->lpszText = MAKEINTRESOURCE(IDS_VIEW_BANANAS);		break;
							case IDM_VIEW_CAMERAS:	nmtt->lpszText = MAKEINTRESOURCE(IDS_VIEW_CAMERAS);		break;
							case IDM_LEVEL_SELECT:	nmtt->lpszText = MAKEINTRESOURCE(IDS_LEVEL_SELECT);		break;
							case IDM_LEVEL_SETTING:	nmtt->lpszText = MAKEINTRESOURCE(IDS_LEVEL_SETTING);	break;
							case IDM_MAP_TILEMAP:	nmtt->lpszText = MAKEINTRESOURCE(IDS_MAP_TILEMAP);		break;
							case IDM_FILE_TESTPLAY:	nmtt->lpszText = MAKEINTRESOURCE(IDS_FILE_TESTPLAY);	break;
						}
					}
					return 0;
				case TBN_DROPDOWN:
					if (nmhdr->idFrom == IDC_TOOL0){
						LPNMTOOLBAR nmtb = (LPNMTOOLBAR)nmhdr;
						HMENU hMenu = NULL;
						switch (nmtb->iItem){
							case IDM_VIEW_TILEMAP:	hMenu = GetSubMenu(GetSubMenu(GetMenu(hWnd), 2), 3);	break;
							case IDM_VIEW_SPRITES:	hMenu = GetSubMenu(GetSubMenu(GetMenu(hWnd), 2), 4);	break;
							case IDM_VIEW_BANANAS:	hMenu = GetSubMenu(GetSubMenu(GetMenu(hWnd), 2), 5);	break;
							case IDM_LEVEL_SELECT:	hMenu = GetSubMenu(GetSubMenu(GetMenu(hWnd), 3), 1);	break;
						}
						if (!hMenu) break;
						RECT rt;
						SendMessage(hwTool, TB_GETRECT, (WPARAM)nmtb->iItem, (LPARAM)&rt);
						MapWindowPoints(hwTool, HWND_DESKTOP, (LPPOINT)&rt, 2);
						TPMPARAMS tpm;
						tpm.cbSize = sizeof tpm;
						tpm.rcExclude = rt;
						TrackPopupMenuEx(hMenu, TPM_VERTICAL, rt.left, rt.bottom, hWnd, &tpm);
						return 0;
					}
					break;
			}
			break;
		case WM_COMMAND:
			if (command(wParam, lParam)) return 0;
			break;
		case WM_SETFOCUS:
			SetFocus(levels.get_hWnd());
			return 0;
		case WM_DROPFILES:
			open_dropped_files((HDROP)wParam);
			return 0;
		case WM_CLOSE:
			quit();
			return 0;
		case WM_DESTROY:
			recent_files.save_data();
			save_window_pos();
			PostQuitMessage(0);
			return 0;
	}
	return DefWindowProc(hWnd, msg, wParam, lParam);
}

void ProjectManager::init_toolbar(){
	SetWindowLong(hwTool, GWL_STYLE, (LONG)( (DWORD)GetWindowLong(hwTool, GWL_STYLE) | TBSTYLE_FLAT ));
	SendMessage(hwTool, TB_SETEXTENDEDSTYLE, 0, (LPARAM)TBSTYLE_EX_DRAWDDARROWS);
	SendMessage(hwTool, TB_BUTTONSTRUCTSIZE, (WPARAM)(sizeof(TBBUTTON)), 0);
	TBADDBITMAP tbab;
	tbab.hInst		= HINST_COMMCTRL;
	tbab.nID		= IDB_STD_SMALL_COLOR;
	int index_std	= SendMessage(hwTool, TB_ADDBITMAP, 0, (LPARAM)&tbab);
	tbab.hInst		= hInst;
	tbab.nID		= IDB_TOOLIMG;
	int index_mybmp	= SendMessage(hwTool, TB_ADDBITMAP, 8, (LPARAM)&tbab);
	TBBUTTON buttons[18] = {
		{ 0,							0,					0,					TBSTYLE_SEP, 0, 0, 0 },		// -------
		{ index_std + STD_FILEOPEN,		IDM_FILE_OPEN,		TBSTATE_ENABLED,	TBSTYLE_BUTTON, 0, 0, 0 },	// J
		{ index_std + STD_FILESAVE,		IDM_FILE_SAVE,		TBSTATE_ENABLED,	TBSTYLE_BUTTON, 0, 0, 0 },	// ۑ
		{ 0,							0,					0,					TBSTYLE_SEP, 0, 0, 0 },		// -------
		{ index_std + STD_UNDO,			IDM_EDIT_UNDO,		TBSTATE_ENABLED,	TBSTYLE_BUTTON, 0, 0, 0 },	// ɖ߂
		{ index_std + STD_REDOW,		IDM_EDIT_REDO,		TBSTATE_ENABLED,	TBSTYLE_BUTTON, 0, 0, 0 },	// 蒼
		{ 0,							0,					0,					TBSTYLE_SEP, 0, 0, 0 },		// -------
		{ index_mybmp + 0,				IDM_VIEW_TILEMAP,	TBSTATE_ENABLED,	TBSTYLE_DROPDOWN, 0, 0, 0 },// n`\
		{ index_mybmp +	1,				IDM_VIEW_SPRITES,	TBSTATE_ENABLED,	TBSTYLE_DROPDOWN, 0, 0, 0 },// XvCg\
		{ index_mybmp + 2,				IDM_VIEW_BANANAS,	TBSTATE_ENABLED,	TBSTYLE_DROPDOWN, 0, 0, 0 },// oii\
		{ index_mybmp + 3,				IDM_VIEW_CAMERAS,	TBSTATE_ENABLED,	TBSTYLE_CHECK, 0, 0, 0 },	// J\
		{ 0,							0,					0,					TBSTYLE_SEP, 0, 0, 0 },		// -------
		{ index_mybmp + 4,				IDM_LEVEL_SELECT,	TBSTATE_ENABLED,	TBSTYLE_DROPDOWN, 0, 0, 0 },// Xe[WI
		{ index_mybmp + 5,				IDM_LEVEL_SETTING,	TBSTATE_ENABLED,	TBSTYLE_BUTTON, 0, 0, 0 },	// Xe[Wݒ
		{ 0,							0,					0,					TBSTYLE_SEP, 0, 0, 0 },		// -------
		{ index_mybmp + 6,				IDM_MAP_TILEMAP,	TBSTATE_ENABLED,	TBSTYLE_BUTTON, 0, 0, 0 },	// }bv̒n`f[^Ȃ
		{ 0,							0,					0,					TBSTYLE_SEP, 0, 0, 0 },		// -------
		{ index_mybmp + 7,				IDM_FILE_TESTPLAY,	TBSTATE_ENABLED,	TBSTYLE_BUTTON, 0, 0, 0 },	// eXgvC
	};
	SendMessage(hwTool, TB_ADDBUTTONS, (WPARAM)18, (LPARAM)buttons);
	SendMessage(hwTool, TB_AUTOSIZE, 0, 0);

	return;
}

void ProjectManager::nc_activate(WPARAM wParam){
	SetWindowPos(hWnd, HWND_TOP, 0, 0, 0, 0,
		SWP_NOMOVE | SWP_NOSIZE | SWP_NOACTIVATE);
	//DefWindowProc(hWnd, WM_NCACTIVATE, wParam, 0);
	return;
}

void ProjectManager::changed_normal(){
	if (bChangedInrev) return;
	if (nChangeStep >= 0){
		nChangeStep++;			// Xebv(t)
	}else{
		bChangedInrev = true;	// st
	}
	refresh_caption();
	return;
}

void ProjectManager::changed_undo(){
	if (bChangedInrev) return;
	nChangeStep--;
	refresh_caption();
	return;
}

void ProjectManager::changed_redo(){
	if (bChangedInrev) return;
	nChangeStep++;
	refresh_caption();
	return;
}

BOOL ProjectManager::command(WPARAM wParam, LPARAM lParam){
	if (recent_files.command(wParam, lParam)) return TRUE;
	if (test_play.command(wParam, lParam)) return TRUE;
	if (bEnabled){
		if (undo_buffer.command(wParam, lParam)) return TRUE;
		if (levels.command(wParam, lParam)) return TRUE;
		if (isd.command(wParam, lParam)) return TRUE;
		if (maps.command(wParam, lParam)) return TRUE;
		if (sprites.command(wParam, lParam)) return TRUE;
		if (areas.command(wParam, lParam)) return TRUE;
	}
	switch (LOWORD(wParam)){
		case IDM_FILE_OPEN:
			open();
			return TRUE;
		case IDM_FILE_SAVE:
			if (bEnabled) save();
			return TRUE;
		case IDM_FILE_SAVEAS:
			if (bEnabled) save_as();
			return TRUE;
		case IDM_FILE_RELOAD:
			if (bEnabled) reload();
			return TRUE;
		case IDM_FILE_CLOSE:
			if (bEnabled) close();
			return TRUE;
		case IDM_FILE_QUIT:
			SendMessage(hWnd, WM_CLOSE, 0, 0);
			return TRUE;
		case IDM_VIEW_TOOL:
			{
				RECT rt;
				GetClientRect(hWnd, &rt);
				bShowTool = (bShowTool? FALSE : TRUE);
				ShowWindow(hwTool, bShowTool? SW_SHOW : SW_HIDE);
				SendMessage(hWnd, WM_SIZE, 0, (LPARAM)((DWORD)(WORD)(rt.right-rt.left) | ((DWORD)(WORD)(rt.bottom-rt.top)<<16)));
				MENUITEMINFO mi;
				mi.cbSize	= sizeof mi;
				mi.fMask	= MIIM_STATE;
				mi.fState	= bShowTool? MFS_CHECKED : MFS_UNCHECKED;
				SetMenuItemInfo(GetMenu(hWnd), IDM_VIEW_TOOL, FALSE, &mi);
			}
			return TRUE;
		case IDM_VIEW_STATUS:
			{
				RECT rt;
				GetClientRect(hWnd, &rt);
				bShowStatus = (bShowStatus? FALSE : TRUE);
				ShowWindow(hwStatus, bShowStatus? SW_SHOW : SW_HIDE);
				SendMessage(hWnd, WM_SIZE, 0, (LPARAM)((DWORD)(WORD)(rt.right-rt.left) | ((DWORD)(WORD)(rt.bottom-rt.top)<<16)));
				MENUITEMINFO mi;
				mi.cbSize	= sizeof mi;
				mi.fMask	= MIIM_STATE;
				mi.fState	= bShowStatus? MFS_CHECKED : MFS_UNCHECKED;
				SetMenuItemInfo(GetMenu(hWnd), IDM_VIEW_STATUS, FALSE, &mi);
			}
			return TRUE;
	}
	return FALSE;
}

BOOL ProjectManager::translate_dialog_message(MSG *msg){
	if (levels.translate_dialog_message(msg)) return TRUE;
	if (sprites.translate_dialog_message(msg)) return TRUE;
	if (areas.translate_dialog_message(msg)) return TRUE;
	return FALSE;
}

void ProjectManager::open(){
	if (!query_saving()) return;
	if (bEnabled){					// ݂̃vWFNgĂJB
		undo_buffer.clear();		// A{͐VvWFNgǂݍ݂ɐĂׂB
		levels.clear();
		set_state(false);
	}
	OPENFILENAME ofn;
	char _filename[MAX_PATH];
	char _filetitle[256];
	ZeroMemory(&ofn, sizeof ofn);
	ZeroMemory(_filename, sizeof _filename);
	ZeroMemory(_filetitle, sizeof _filetitle);
	ofn.lStructSize			= sizeof ofn;
	ofn.hwndOwner			= hWnd;
	ofn.hInstance			= NULL;
	ofn.lpstrFilter			= "SNES ROM(*.smc;*.sfc)\0*.smc;*.sfc\0All Files(*.*)\0*.*\0\0";
	ofn.lpstrFile			= _filename;
	ofn.nMaxFile			= sizeof _filename;
	ofn.lpstrFileTitle		= _filetitle;
	ofn.nMaxFileTitle		= sizeof _filetitle;
	ofn.Flags				= OFN_FILEMUSTEXIST;
	ofn.lpstrDefExt			= "smc";
	ofn.lpstrTitle			= "Open";
	if (!GetOpenFileName(&ofn)){
		return;
	}
	strcpy_s(filename, sizeof filename, _filename);
	strcpy_s(filetitle, sizeof filetitle, _filetitle);
	load();
	return;
}

void ProjectManager::open_cmdline(LPCSTR cmdline){
	int i=0, j=0, bQuoted=FALSE, bSJIS=FALSE, c;
	while (c = cmdline[i]){
		if (bQuoted){
			if (!bSJIS && c == '"'){
				bQuoted = FALSE;
				filename[j] = '\0';
				strcpy_s(filetitle, sizeof filetitle, PathFindFileName(filename));
				if (load()) return;
			}else{
				filename[j++] = c;
				bSJIS = (!bSJIS) && ( ( c >= 0x81 && c <= 0x9F ) || ( c >= 0xE0 && c <= 0xFC ) );
			}
		}else{
			if (!bSJIS && c == '"'){
				if (j != 0){
					filename[j] = '\0';
					strcpy_s(filetitle, sizeof filetitle, PathFindFileName(filename));
					if (load()) return;
					j = 0;
				}
				bQuoted = TRUE;
			}else if (!bSJIS && c == ' '){
				if (j != 0){
					filename[j] = '\0';
					strcpy_s(filetitle, sizeof filetitle, PathFindFileName(filename));
					if (load()) return;
				}
			}else{
				filename[j++] = c;
				bSJIS = (!bSJIS) && ( ( c >= 0x81 && c <= 0x9F ) || ( c >= 0xE0 && c <= 0xFC ) );
			}
		}
		i++;
	}
	if (j != 0){
		filename[j] = '\0';
		strcpy_s(filetitle, sizeof filetitle, PathFindFileName(filename));
		if (load()) return;
	}
	return;
}

void ProjectManager::open_dropped_files(HDROP hDrop){
	if (!query_saving()) return;
	if (bEnabled){					// ݂̃vWFNgĂJB
		undo_buffer.clear();		// A{͐VvWFNgǂݍ݂ɐĂׂB
		levels.clear();
		set_state(false);
	}
	UINT nFiles = DragQueryFile(hDrop, 0xFFFFFFFF, NULL, 0);
	for (UINT i=0; i<nFiles; i++){
		DragQueryFile(hDrop, i, filename, sizeof filename);
		strcpy_s(filetitle, sizeof filetitle, PathFindFileName(filename));
		if (load()) goto open_dropped_files_finished;
	}
open_dropped_files_finished:
	DragFinish(hDrop);
	return;
}

void ProjectManager::load_file(LPCSTR _filename){
	if (!query_saving()) return;
	if (bEnabled){					// ݂̃vWFNgĂJB
		undo_buffer.clear();		// A{͐VvWFNgǂݍ݂ɐĂׂB
		levels.clear();
		set_state(false);
	}
	strcpy_s(filename, sizeof filename, _filename);
	strcpy_s(filetitle, sizeof filetitle, PathFindFileName(filename));
	load();
	return;
}

bool ProjectManager::load(){
	HANDLE hFile;
	DWORD size, size_read;
	BYTE *buf_org_without_header;
	char CartName[256];
	int sz;
	recent_files.file_opened(filename);	// ֒ǉ
	hFile = CreateFile(filename, GENERIC_READ, FILE_SHARE_READ, NULL,
		OPEN_EXISTING, FILE_ATTRIBUTE_NORMAL, NULL);
	if (hFile == INVALID_HANDLE_VALUE) return false;
	size = GetFileSize(hFile, NULL);
	// wb_̗Lʂēǂݍ
	if (size == ROM_SIZE_NO_HEADER){
		bHasHeader = false;
		buf_org_without_header = new BYTE[ROM_SIZE_NO_HEADER];
		ReadFile(hFile, buf_org_without_header, ROM_SIZE_NO_HEADER, &size_read, NULL);
	}else if (size == ROM_SIZE_NO_HEADER + ROM_HEADER_SIZE){
		bHasHeader = true;
		buf_org_without_header = new BYTE[ROM_SIZE_NO_HEADER];
		ReadFile(hFile, header, ROM_HEADER_SIZE, &size_read, NULL);
		ReadFile(hFile, buf_org_without_header, ROM_SIZE_NO_HEADER, &size_read, NULL);
	}else{
		MessageBox(hWnd, "Invalid File", "Error", MB_OK | MB_ICONEXCLAMATION);
		CloseHandle(hFile);
		return false;
	}
	CloseHandle(hFile);
	// tH[}bgʂēǂݍ
	GetPrivateProfileString("ROMInfo", "CartName", "SUPER DONKEY KONG 2",
		CartName, sizeof CartName, RomInfoFileName);
	sz = strlen(CartName);
	bInterleaved = false;
	for (int i=0; i<sz; i++){
		if (buf_org_without_header[0xFFC0 + i] != CartName[i]){
			bInterleaved = true;
			break;
		}
	}
	if (bInterleaved){
		for (int i=0; i<sz; i++){
			if (buf_org_without_header[0x7FC0 + i] != CartName[i]){
				MessageBox(hWnd, "ROM cannot be opened.\r\nPlease check that your ROM is the correct region and does not have a header.",
					"Error", MB_OK | MB_ICONEXCLAMATION);
				delete[] buf_org_without_header;
				return false;
			}
		}
	}
	if (bInterleaved){
		if (buf == NULL) buf = new BYTE[ROM_SIZE_NO_HEADER];
		for (int i=0; i<0x40; i++){
			memcpy_s(buf + 0x010000*i, 0x8000, buf_org_without_header + 0x008000*i + 0x200000, 0x8000);
			memcpy_s(buf + 0x010000*i + 0x008000, 0x8000, buf_org_without_header + 0x008000*i, 0x8000);
		}
		delete[] buf_org_without_header;
	}else{
		if (buf != NULL) delete[] buf;
		buf = buf_org_without_header;
	}
	// Ĵɐ
	set_state(true);
	nChangeStep = 0;
	bChangedInrev = false;
	// Ŏqvf̏
	undo_buffer.clear();
	maps.load();
	isd.load();
	sprites.load();
	bananas.load();
	levels.reset();		// Xe[W̓ǂݍ݂̓}bvEX^CEXvCgȂǂɈˑ̂ōŌ
	return true;
}

void ProjectManager::save(){
	HANDLE hFile;
	DWORD size_written;
	hFile = CreateFile(filename, GENERIC_WRITE, FILE_SHARE_READ, NULL,
		CREATE_ALWAYS, FILE_ATTRIBUTE_NORMAL, NULL);
	if (hFile == INVALID_HANDLE_VALUE) return;
	// wb_[Ώ
	if (bHasHeader){
		WriteFile(hFile, header, ROM_HEADER_SIZE, &size_written, NULL);
	}
	// ̃tH[}bgɒď
	if (bInterleaved){
		BYTE *buf_org_without_header = new BYTE[ROM_SIZE_NO_HEADER];
		for (int i=0; i<0x40; i++){
			memcpy_s(buf_org_without_header + 0x008000*i + 0x200000, 0x8000, buf + 0x010000*i, 0x8000);
			memcpy_s(buf_org_without_header + 0x008000*i, 0x8000, buf + 0x010000*i + 0x008000, 0x8000);
		}
		WriteFile(hFile, buf_org_without_header, ROM_SIZE_NO_HEADER, &size_written, NULL);
		delete[] buf_org_without_header;
	}else{
		WriteFile(hFile, buf, ROM_SIZE_NO_HEADER, &size_written, NULL);
	}
	SetEndOfFile(hFile);
	CloseHandle(hFile);
	nChangeStep = 0;
	bChangedInrev = false;
	refresh_caption();
	return;
}

void ProjectManager::save_as(){
	OPENFILENAME ofn;
	char _filename[MAX_PATH];
	char _filetitle[256];
	ZeroMemory(&ofn, sizeof ofn);
	strcpy_s(_filename, sizeof _filename, filename);
	strcpy_s(_filetitle, sizeof _filetitle, filetitle);
	ofn.lStructSize			= sizeof ofn;
	ofn.hwndOwner			= hWnd;
	ofn.hInstance			= NULL;
	ofn.lpstrFilter			= "All Files(*.*)\0*.*\0\0";
	ofn.lpstrFile			= _filename;
	ofn.nMaxFile			= sizeof _filename;
	ofn.lpstrFileTitle		= _filetitle;
	ofn.nMaxFileTitle		= sizeof _filetitle;
	ofn.Flags				= OFN_OVERWRITEPROMPT | OFN_EXPLORER;
	ofn.lpstrTitle			= "Save As";
	if (!GetSaveFileName(&ofn)){
		return;
	}
	strcpy_s(filename, sizeof filename, _filename);
	strcpy_s(filetitle, sizeof filetitle, _filetitle);
	set_state(true);
	save();
	return;
}

void ProjectManager::reload(){
	if (bChangedInrev || nChangeStep!=0){
		int r = MessageBox(hWnd, "All unsaved changes will be lost, are you sure?",
			"SDK2 Editor", MB_OKCANCEL | MB_ICONEXCLAMATION);
		if (r == IDCANCEL) return;
	}
	undo_buffer.clear();
	levels.clear();
	load();
	return;
}

void ProjectManager::close(){
	if (!query_saving()) return;
	undo_buffer.clear();
	levels.clear();
	set_state(false);
	return;
}

void ProjectManager::quit(){
	if (!query_saving()) return;
	destroy();
	return;
}

bool ProjectManager::query_saving(){
	if (bEnabled && (bChangedInrev || nChangeStep!=0)){
		char s[1024];
		sprintf_s(s, sizeof s, "%s has been changed. Do you want to save the changes?", filetitle);
		int r = MessageBox(hWnd, s, "SDK2 Editor", MB_ICONEXCLAMATION | MB_YESNOCANCEL);
		if (r == IDCANCEL) return false;
		if (r == IDYES){
			save();
		}
	}
	return true;
}



