
///////////////////////////////////////////////////////////////////////////////
//
//	recent_files.cpp
//

#include <stdio.h>
#include <shlwapi.h>
#include "project.h"
#include "id.h"


RecentFileManager::RecentFileManager(ProjectManager *_project)
: project(_project)
{}

BOOL RecentFileManager::create(){
	char str[16];
	char filename[MAX_PATH];
	HMENU hMenu = GetSubMenu(GetSubMenu(GetMenu(project->get_hWnd()), 0), 1);
	MENUITEMINFO mi;
	mi.cbSize		= sizeof mi;
	mi.fMask		= MIIM_TYPE | MIIM_ID;
	mi.fType		= MFT_STRING;
	for (int i=0; i<FILES_MAX; i++){
		sprintf_s(str, sizeof str, "%d", i);
		GetPrivateProfileString("RecentFiles", str, "", filename, sizeof filename, ConfigFileName);
		if (filename[0] == '\0') break;
		files.push_back(std::string(filename));
	}
	Filenames::iterator i, i_end=files.end();
	int n;
	for (i=files.begin(), n=0; i!=i_end; i++, n++){
		sprintf_s(filename, sizeof filename, "&%d %s", n, (*i).c_str());
		mi.wID			= IDM_FILE_RECENT_BEGIN + n;
		mi.dwTypeData	= filename;
		mi.cch			= strlen(filename);
		InsertMenuItem(hMenu, n, TRUE, &mi);
	}
	return TRUE;
}

void RecentFileManager::save_data(){
	char str[16];
	Filenames::iterator i, i_end=files.end();
	int n;
	for (i=files.begin(), n=0; i!=i_end; i++, n++){
		sprintf_s(str, sizeof str, "%d", n);
		WritePrivateProfileString("RecentFiles", str, (*i).c_str(), ConfigFileName);
	}
	return;
}

void RecentFileManager::file_opened(LPCSTR filename){
	char text[MAX_PATH];
	HMENU hMenu = GetSubMenu(GetSubMenu(GetMenu(project->get_hWnd()), 0), 1);
	MENUITEMINFO mi;
	int nFiles = files.size();
	for (int i=0; i<nFiles; i++){
		DeleteMenu(hMenu, 0, MF_BYPOSITION);
	}
	std::string filename_str = filename;
	Filenames::iterator i, i_end = files.end();
	int n;
	for (i=files.begin(), n=0; i!=i_end; i++, n++){
		if (PathMatchSpec(filename, (*i).c_str())){
			files.erase(i);
			break;
		}
	}
	if (files.size() >= FILES_MAX){
		files.pop_back();
	}
	files.push_front(filename_str);
	mi.cbSize		= sizeof mi;
	mi.fMask		= MIIM_TYPE | MIIM_ID;
	mi.fType		= MFT_STRING;
	for (i=files.begin(), i_end=files.end(), n=0; i!=i_end; i++, n++){
		sprintf_s(text, sizeof text, "%d %s", n, (*i).c_str());
		mi.wID			= IDM_FILE_RECENT_BEGIN + n;
		mi.dwTypeData	= text;
		mi.cch			= strlen(text);
		InsertMenuItem(hMenu, n, TRUE, &mi);
	}
	return;
}

void RecentFileManager::clear(){
	char str[16];
	HMENU hMenu = GetSubMenu(GetSubMenu(GetMenu(project->get_hWnd()), 0), 1);
	Filenames::iterator i, i_end=files.end();
	int n;
	for (i=files.begin(), n=0; i!=i_end; i++, n++){
		DeleteMenu(hMenu, 0, MF_BYPOSITION);
	}
	for (int i=0; i<FILES_MAX; i++){
		sprintf_s(str, sizeof str, "%d", i);
		WritePrivateProfileString("RecentFiles", str, "", ConfigFileName);
	}
	files.clear();
	return;
}

BOOL RecentFileManager::command(WPARAM wParam, LPARAM lParam){
	WORD cmdID = LOWORD(wParam);
	if (cmdID == IDM_FILE_RECENT_CLEAR){
		clear();
		return TRUE;
	}
	if (cmdID >= IDM_FILE_RECENT_BEGIN && cmdID < IDM_FILE_RECENT_CLEAR){
		project->load_file(files.at(cmdID-IDM_FILE_RECENT_BEGIN).c_str());
	}
	return FALSE;
}







