
///////////////////////////////////////////////////////////////////////////////
//
//	rom_rw.cpp
//

#include <stdio.h>
#include "project.h"

bool ProjectManager::read_data(void *dest, size_t nSize, LADRS adrs) const{
	int offset = adrs.get();
	if (offset + nSize > ROM_SIZE_NO_HEADER) return false;
	memcpy_s(dest, nSize, buf+offset, nSize);
	return true;
}
bool ProjectManager::read_data(void *dest, size_t nSize, BANK bank, SADRS adrs) const{
	int offset = adrs | (bank << 16);
	if (offset + nSize > ROM_SIZE_NO_HEADER) return false;
	memcpy_s(dest, nSize, buf+offset, nSize);
	return true;
}

SADRS ProjectManager::read_sadrs(LADRS adrs) const{
	int offset = adrs.get();
	return *(SADRS*)(buf + offset);
}
SADRS ProjectManager::read_sadrs(BANK bank, SADRS adrs) const{
	int offset = adrs | (bank << 16);
	return *(SADRS*)(buf + offset);
}

LADRS ProjectManager::read_ladrs(LADRS adrs) const{
	int offset = adrs.get();
	return *(DWORD*)(buf + offset);
}
LADRS ProjectManager::read_ladrs(BANK bank, SADRS adrs) const{
	int offset = adrs | (bank << 16);
	return *(DWORD*)(buf + offset);
}

BYTE ProjectManager::read_byte(LADRS adrs) const{
	int offset = adrs.get();
	return *(BYTE *)(buf + offset);
}
BYTE ProjectManager::read_byte(BANK bank, SADRS adrs) const{
	int offset = adrs | (bank << 16);
	return *(BYTE *)(buf + offset);
}

WORD ProjectManager::read_word(LADRS adrs) const{
	int offset = adrs.get();
	return *(WORD *)(buf + offset);
}
WORD ProjectManager::read_word(BANK bank, SADRS adrs) const{
	int offset = adrs | (bank << 16);
	return *(WORD *)(buf + offset);
}


bool ProjectManager::write_data(const void *src, size_t nSize, LADRS adrs){
	int offset = adrs.get();
	if (offset + nSize > ROM_SIZE_NO_HEADER) return false;
	memcpy_s(buf+offset, nSize, src, nSize);
	return true;
}
bool ProjectManager::write_data(const void *src, size_t nSize, BANK bank, SADRS adrs){
	int offset = adrs | (bank << 16);
	if (offset + nSize > ROM_SIZE_NO_HEADER) return false;
	memcpy_s(buf+offset, nSize, src, nSize);
	return true;
}

void ProjectManager::write_sadrs(SADRS data, LADRS adrs){
	int offset = adrs.get();
	*(SADRS*)(buf + offset) = data;
	return;
}
void ProjectManager::write_sadrs(SADRS data, BANK bank, SADRS adrs){
	int offset = adrs | (bank << 16);
	*(SADRS*)(buf + offset) = data;
	return;
}

void ProjectManager::write_ladrs(LADRS data, LADRS adrs){
	int offset = adrs.get();
	*(WORD*)(buf + offset) = data.adrs;
	*(BYTE*)(buf + offset + 2) = data.bank;
	return;
}
void ProjectManager::write_ladrs(LADRS data, BANK bank, SADRS adrs){
	int offset = adrs | (bank << 16);
	*(WORD*)(buf + offset) = data.adrs;
	*(BYTE*)(buf + offset + 2) = data.bank;
	return;
}

void ProjectManager::write_byte(BYTE data, LADRS adrs){
	int offset = adrs.get();
	*(BYTE*)(buf + offset) = data;
	return;
}
void ProjectManager::write_byte(BYTE data, BANK bank, SADRS adrs){
	int offset = adrs | (bank << 16);
	*(BYTE*)(buf + offset) = data;
	return;
}

void ProjectManager::write_word(WORD data, LADRS adrs){
	int offset = adrs.get();
	*(WORD*)(buf + offset) = data;
	return;
}
void ProjectManager::write_word(WORD data, BANK bank, SADRS adrs){
	int offset = adrs | (bank << 16);
	*(WORD*)(buf + offset) = data;
	return;
}

void *ProjectManager::get_pointer(LADRS adrs) const{
	return buf + adrs.get();
}

void *ProjectManager::get_pointer(BANK bank, SADRS adrs) const{
	int offset = adrs | (bank << 16);
	return buf + offset;
}



