
///////////////////////////////////////////////////////////////////////////////
//
//	sprite_edit.h
//

#ifndef _SPRITE_EDIT_
#define _SPRITE_EDIT_

///////////////////////////////////////////////////////////////////////////////

#include <list>
#include <windows.h>
#include "base.h"
#include "window.h"


class ProjectManager;
class SpritesManager;
class EditsprManager;
class SpriteDataIterator;




class EditsprDataPage_CommandBase : public MyDialog{
public:
	virtual ~EditsprDataPage_CommandBase(){}
};



//	EditsprDataPage: XvCgf[^ҏW̃y[W
//---------------------------------------------------------
class EditsprDataPage : public MyDialog{

private:
	ProjectManager *project;
	SpritesManager *sprites;
	EditsprManager *editspr;
	SADRS adrs;
	EditsprDataPage_CommandBase *command_page;
	bool unable_edit_change;

private:
	LPCSTR get_template() const{ return "SPRITE_EDIT_DATA_PAGE"; }
	BOOL Proc(HWND hDlg, UINT msg, WPARAM wParam, LPARAM lParam);
	void init();
	void close();
	//void size(int width, int height);
	void edit_change();
	void command_apply();
	EditsprDataPage_CommandBase *set_command_page(const SpriteDataIterator &itr);

public:
	EditsprDataPage(ProjectManager *_project, SpritesManager *_sprites, EditsprManager *_editspr, SADRS _adrs);
	~EditsprDataPage();
	SADRS get_adrs() const{ return adrs; }
	void reload();
	EditsprDataPage_CommandBase *sel_change(int index);
	EditsprDataPage_CommandBase *select_command_page(const SpriteDataIterator &itr);
	void update_command_list(const SpriteDataIterator &itr);

};


//	EditsprSpritePage: XvCgAhXҏW̃y[W
//---------------------------------------------------------
class EditsprSpritePage : public MyDialog{

private:
	ProjectManager *project;
	SpritesManager *sprites;
	EditsprManager *editspr;
	int sprite_num;
	bool unable_name_edit_change;
	bool unable_adrs_edit_change;

private:
	LPCSTR get_template() const{ return "SPRITE_EDIT_SPRITE_PAGE"; }
	BOOL Proc(HWND hDlg, UINT msg, WPARAM wParam, LPARAM lParam);
	void init();
	void close();
	//void size(int width, int height);
	void name_edit_change();
	void set_name();
	void adrs_edit_change();
	void set_adrs();
	void open_data();

public:
	EditsprSpritePage(ProjectManager *_project, SpritesManager *_sprites, EditsprManager *_editspr, int _sprite_num);
	~EditsprSpritePage();
	int get_sprite_num() const{ return sprite_num; }
	void update_adrs_edit();
	void focus_on_adrs_edit();
	void reload();

};


//	EditsprManager: XvCgҏW
//---------------------------------------------------------
class EditsprManager : public MyDialog{

	struct TabPage{
		enum PageType{
			SPRITE_PAGE=0,
			DATA_PAGE=1
		} type;
		union{
			EditsprSpritePage *sprite_page;
			EditsprDataPage *data_page;
		};
	};
	typedef std::list<TabPage> PageList;

private:

	ProjectManager *project;
	SpritesManager *sprites;
	PageList pages;

private:

	LPCSTR get_template() const{ return "SPRITE_EDIT"; }
	BOOL Proc(HWND hDlg, UINT msg, WPARAM wParam, LPARAM lParam);
	void init();
	void save_config();
	//void sizing(WPARAM wParam, LPARAM lParam);
	void size(int width, int height);
	void sprite_sel();
	void type_changed();
	void open();
	void tab_changed(int index);
	void insert_tab_page(int index, TabPage page, LPSTR text);
	void delete_tab_page(int index);

public:
	EditsprManager(ProjectManager *_project, SpritesManager *_sprites);
	~EditsprManager();
	//BOOL create();
	BOOL command(WPARAM wParam, LPARAM lParam);
	BOOL translate_dialog_message(MSG *msg);
	void set_state(BOOL bEnabled);
	void reload();

	EditsprSpritePage* open_sprite_page(int sprite_num);
	EditsprDataPage* open_data_page(SADRS adrs);
	void close_sprite_page(EditsprSpritePage *page);
	void close_data_page(EditsprDataPage *page);

};


///////////////////////////////////////////////////////////////////////////////

#endif // _SPRITE_EDIT_




