
///////////////////////////////////////////////////////////////////////////////
//
//	sprites.cpp
//

#include "project.h"
#include "id.h"


#pragma warning(disable: 4355)


SpritesManager::SpritesManager(ProjectManager *_proj)
: project(_proj), findspr(_proj, this), editspr(_proj, this)
{
	sprite_count		= GetPrivateProfileInt("Sprites", "Count", 2083, SpriteInfoFileName);
	sprite_name			= new char[sprite_count][256];
	pSpriteAdrs			= GetPrivateProfileInt("Sprites", "SpriteAddress", 0x3BE800, SpriteInfoFileName);
	pSpriteBank			= GetPrivateProfileInt("Sprites", "SpriteDataBank", 0x3F, SpriteInfoFileName);
	control_code_count	= GetPrivateProfileInt("ControlCode", "Count", 200, SpriteInfoFileName);
	control_code_name	= new char[control_code_count][64];
	for (int i=0; i<sprite_count; i++){
		char s[16];
		sprintf_s(s, sizeof s, "%04X", i*2);	// XvCgԍ = index * 2 ɒӁB
		GetPrivateProfileString("SpriteName", s, "?", sprite_name[i], 256, SpriteInfoFileName);
	}
	for (int i=0; i<control_code_count; i++){
		char s[16];
		sprintf_s(s, sizeof s, "%04X", i*4);	// R[h = index * 4
		GetPrivateProfileString("ControlCodeName", s, "?", control_code_name[i], 64, SpriteInfoFileName);
	}
	sprite_adrs			= new SADRS[sprite_count];
	control_code		= new WORD[sprite_count];
}

SpritesManager::~SpritesManager(){
	delete[] control_code;
	delete[] sprite_adrs;
	delete[] control_code_name;
	delete[] sprite_name;
}



BOOL SpritesManager::create(){
	//if (!findspr.create()) return FALSE;
	//if (!editspr.create()) return FALSE;
	return TRUE;
}

BOOL SpritesManager::command(WPARAM wParam, LPARAM lParam){
	if (findspr.command(wParam, lParam)) return TRUE;
	if (editspr.command(wParam, lParam)) return TRUE;
	return FALSE;
}

BOOL SpritesManager::translate_dialog_message(MSG *msg){
	if (findspr.translate_dialog_message(msg)) return TRUE;
	if (editspr.translate_dialog_message(msg)) return TRUE;
	return FALSE;
}

void SpritesManager::set_state(BOOL bEnabled){
	findspr.set_state(bEnabled);
	editspr.set_state(bEnabled);
	return;
}



void SpritesManager::load(){
	// XvCgAhX̓ǂݍ
	for (int i=0; i<sprite_count; i++){
		sprite_adrs[i] = project->read_sadrs(pSpriteAdrs.bank, pSpriteAdrs.adrs + i*2);
	}
	// XvCgR[h̓ǂݍ
	for (int i=0; i<sprite_count; i++){
		load_control_code(i, sprite_adrs[i]);
	}
	editspr.reload();
	return;
}

void SpritesManager::load_control_code(int sprite_index, SADRS adrs){
	for (SpriteDataIterator sprite(project, pSpriteBank, adrs); sprite; sprite++){
		WORD cmd = sprite.get_command();
		if (cmd == 0x0000){
			control_code[sprite_index] = sprite.get_value();
		}else if (cmd == 0x8300){
			load_control_code(sprite_index, (SADRS)sprite.get_value());
		}
	}
	return;
}


void SpritesManager::save(){
}


BANK SpritesManager::get_sprite_bank(){
	return pSpriteBank;
}

int SpritesManager::get_sprite_count(){
	return sprite_count;
}

const char *SpritesManager::get_sprite_name(WORD sprite_num){
	return sprite_name[sprite_num/2];
}

int SpritesManager::set_sprite_name(WORD sprite_num, const char *name){
	char s[16];
	sprintf_s(s, sizeof s, "%04X", sprite_num);
	WritePrivateProfileString("SpriteName", s, name, SpriteInfoFileName);
	return strcpy_s(sprite_name[sprite_num/2], 256, name);
}

int SpritesManager::get_control_code_count(){
	return control_code_count;
}

const char *SpritesManager::get_control_code_name(WORD control_code){
	return control_code_name[control_code/4];
}




SADRS SpritesManager::get_sprite_adrs(WORD sprite_num){
	return sprite_adrs[sprite_num/2];
}

void SpritesManager::set_sprite_adrs(WORD sprite_num, SADRS adrs){
	sprite_adrs[sprite_num/2] = adrs;
	// ROMɏ
	project->write_sadrs(adrs, pSpriteAdrs.bank, pSpriteAdrs.adrs + sprite_num);
	// R[h̓XvCgAhXɈˑ̂ōēǂݍ
	load_control_code(sprite_num/2, adrs);
	return;
}

SpriteDataIterator SpritesManager::get_sprite(WORD _sprite_adrs){
	return SpriteDataIterator(project, pSpriteBank, _sprite_adrs);
}

WORD SpritesManager::get_control_code(WORD sprite_num){
	return control_code[sprite_num/2];
}


bool SpritesManager::get_sprite_graphics(WORD sprite_num, WORD *p_dynamic_num, WORD *p_palette_num, bool *h_flip, bool *v_flip){
	bool bDynamicFound = false;
	bool bPaletteFound = false;
	*h_flip = *v_flip = false;
	load_sprite_graphics(get_sprite_adrs(sprite_num), p_dynamic_num, p_palette_num,
		h_flip, v_flip, &bDynamicFound, &bPaletteFound);
	return bDynamicFound && bPaletteFound;
}

void SpritesManager::load_sprite_graphics(SADRS adrs, WORD *p_dynamic_num, WORD *p_palette_num,
	bool *h_flip, bool *v_flip, bool *bDynamicFound, bool *bPaletteFound)
{
	for (SpriteDataIterator sprite = get_sprite(adrs); sprite; sprite++){
		WORD cmd = sprite.get_command();
		if (cmd == 0x8D00){							// pbg
			*p_palette_num = sprite.get_value();
			*bPaletteFound = true;
		}else if (cmd == 0x8100){					// OtBbN
			*p_dynamic_num = sprite.get_value();
			*bDynamicFound = true;
		}else if (cmd == 0x8500){					// OtBbNX^C
			WORD value = sprite.get_value();
			*h_flip = (value & 0x4000) != 0;
			*v_flip = (value & 0x8000) != 0;
		}else if (cmd == 0x8300){					// p
			load_sprite_graphics(sprite.get_value(), p_dynamic_num, p_palette_num,
				h_flip, v_flip, bDynamicFound, bPaletteFound);
		}
	}
	return;
}







//=============================================================================

SpriteDataIterator::SpriteDataIterator(ProjectManager *_proj, BANK _bank, SADRS _adrs)
: project(_proj), bank(_bank), adrs(_adrs)
{}

SpriteDataIterator::SpriteDataIterator(SpritesManager *sprites, SADRS _adrs)
: project(sprites->get_project()), bank(sprites->get_sprite_bank()), adrs(_adrs)
{}

SpriteDataIterator::SpriteDataIterator(const SpriteDataIterator &_src)
: project(_src.project), bank(_src.bank), adrs(_src.adrs)
{}

SpriteDataIterator::~SpriteDataIterator(){
}

WORD SpriteDataIterator::get_adrs() const{
	return adrs;
}

WORD SpriteDataIterator::get_command() const{
	return project->read_word(bank, adrs);
}

WORD SpriteDataIterator::get_value() const{
	return project->read_word(bank, adrs + 2);
}

WORD SpriteDataIterator::get_value_of13(int index) const{
	return project->read_word(bank, adrs + 2 + index*2);
}
WORD SpriteDataIterator::get_value_of4(int index) const{
	return project->read_word(bank, adrs + 2 + index*2);
}

void SpriteDataIterator::set_command(WORD cmd) const{
	project->write_word(cmd, bank, adrs);
}

void SpriteDataIterator::set_value(WORD val) const{
	project->write_word(val, bank, adrs + 2);
}

void SpriteDataIterator::set_value_of13(WORD val, int index) const{
	project->write_word(val, bank, adrs + 2 + index*2);
}
void SpriteDataIterator::set_value_of4(WORD val, int index) const{
	project->write_word(val, bank, adrs + 2 + index*2);
}

SpriteDataIterator SpriteDataIterator::operator ++(int dummy){
	SpriteDataIterator i(*this);
	switch (get_command()){
		case 0x8A00: adrs += 28; 	break;
		case 0x8E00: adrs += 10;	break;
		default:	 adrs += 4;		break;
	}
	return i;
}
SpriteDataIterator &SpriteDataIterator::operator ++(){
	switch (get_command()){
		case 0x8A00: adrs += 28; 	break;
		case 0x8E00: adrs += 10;	break;
		default:	 adrs += 4;		break;
	}
	return *this;
}

SpriteDataIterator::operator bool(){
	return get_command() != 0x8000;
}

bool SpriteDataIterator::operator ==(const SpriteDataIterator &itr){
	return adrs == itr.adrs;
}

bool SpriteDataIterator::operator !=(const SpriteDataIterator &itr){
	return adrs != itr.adrs;
}



