
///////////////////////////////////////////////////////////////////////////////
//
//	test_play.cpp
//

#include <stdio.h>
#include <shlwapi.h>
#include "id.h"
#include "project.h"




TestPlayManager::TestPlayManager(ProjectManager *_project) : project(_project){
}

BOOL TestPlayManager::command(WPARAM wParam, LPARAM lParam){
	switch (LOWORD(wParam)){
		case IDM_FILE_TESTPLAY:
			test_play();
			return TRUE;
		case IDM_FILE_SETEMU:
			set_emulator();
			return TRUE;
	}
	return FALSE;
}

void TestPlayManager::test_play(){
	char eml_path[MAX_PATH];
	char eml_dirname[MAX_PATH];
	GetPrivateProfileString("Emulator", "Path", "", eml_path, sizeof eml_path, ConfigFileName);
	if (eml_path[0] == '\0'){
		MessageBox(project->get_hWnd(), "Emulator is not set.",
			"Warning", MB_OK | MB_ICONEXCLAMATION);
		return;
	}
	strcpy_s(eml_dirname, sizeof eml_dirname, eml_path);
	char *last_sep = strrchr(eml_dirname, '\\');
	if (last_sep) *last_sep = '\0';
	if (!ShellExecute(NULL, "open", eml_path, project->filename, eml_dirname, SW_SHOWNORMAL)){
		MessageBox(project->get_hWnd(), "Failed to start the Emulator",
			"Warning", MB_OK | MB_ICONEXCLAMATION);
		return;
	}
	return;
}


class SetEmulatorDialog : public MyDialog{
	virtual LPCSTR get_template() const{ return "SET_EMULATOR"; }
	virtual BOOL Proc(HWND hDlg, UINT msg, WPARAM wParam, LPARAM lParam){
		switch (msg){
			case WM_INITDIALOG:
				SetDlgItemText(hDlg, IDC_EDIT0, filename);
				return TRUE;
			case WM_COMMAND:
				switch (LOWORD(wParam)){
					case IDOK:
						GetDlgItemText(hDlg, IDC_EDIT0, filename, nMaxSize);
						EndDialog(hDlg, IDOK);
						return TRUE;
					case IDCANCEL:
						EndDialog(hDlg, IDCANCEL);
						return TRUE;
					case IDC_BUTTON0:
						browse();
						return TRUE;
				}
				break;
		}
		return FALSE;
	}
	LPSTR filename;
	int nMaxSize;
	void browse(){
		char filename_new[MAX_PATH];
		char filetitle[128];
		strcpy_s(filename_new, sizeof filename_new, filename);
		OPENFILENAME ofn;
		ZeroMemory(&ofn, sizeof ofn);
		ofn.lStructSize		= sizeof(OPENFILENAME);
		ofn.hwndOwner		= hWnd;
		ofn.lpstrFilter		= "Emulator(*.exe)\0*.exe\0\0";
		ofn.lpstrFile		= filename_new;
		ofn.nMaxFile		= sizeof filename_new;
		ofn.lpstrFileTitle	= filetitle;
		ofn.nMaxFileTitle	= sizeof filetitle;
		ofn.Flags			= OFN_FILEMUSTEXIST | OFN_HIDEREADONLY;
		ofn.lpstrDefExt		= "exe";
		ofn.lpstrTitle		= "Emulator Settings";
		GetOpenFileName(&ofn);
		SetDlgItemText(hWnd, IDC_EDIT0, filename_new);
		return;
	}
public:
	SetEmulatorDialog(LPSTR _filename, int _nMaxSize) : filename(_filename), nMaxSize(_nMaxSize){}
};



void TestPlayManager::set_emulator(){
	char eml_path[MAX_PATH];
	GetPrivateProfileString("Emulator", "Path", "", eml_path, sizeof eml_path, ConfigFileName);
	SetEmulatorDialog dialog(eml_path, sizeof eml_path);
	int r = dialog.dialog_box(project);
	if (r == IDCANCEL) return;
	WritePrivateProfileString("Emulator", "Path", eml_path, ConfigFileName);
	return;
}








